@@ -42,8 +42,6 @@
 #include "nsGenericHTMLElement.h"
 #include "nsILink.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIDOMStyleSheet.h"
@@ -52,7 +50,6 @@
 #include "nsStyleLinkElement.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
-#include "nsHTMLUtils.h"
 #include "nsIURL.h"
 #include "nsNetUtil.h"
 #include "nsIDocument.h"
@@ -60,8 +57,9 @@
 #include "nsIDOMDocumentEvent.h"
 #include "nsIDOMEventTarget.h"
 #include "nsParserUtils.h"
+#include "nsIPrivateDOMEvent.h"
 
-class nsHTMLLinkElement : public nsGenericHTMLLeafElement,
+class nsHTMLLinkElement : public nsGenericHTMLElement,
                           public nsIDOMHTMLLinkElement,
                           public nsILink,
                           public nsStyleLinkElement
@@ -74,13 +72,13 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLLeafElement::)
+  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
-  NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLLeafElement::)
+  NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
 
   // nsIDOMHTMLElement
-  NS_FORWARD_NSIDOMHTMLELEMENT(nsGenericHTMLLeafElement::)
+  NS_FORWARD_NSIDOMHTMLELEMENT(nsGenericHTMLElement::)
 
   // nsIDOMHTMLLinkElement
   NS_DECL_NSIDOMHTMLLINKELEMENT
@@ -88,110 +86,32 @@ public:
   // nsILink
   NS_IMETHOD    GetLinkState(nsLinkState &aState);
   NS_IMETHOD    SetLinkState(nsLinkState aState);
-  NS_IMETHOD    GetHrefCString(char* &aBuf);
+  NS_IMETHOD    GetHrefURI(nsIURI** aURI);
 
-  NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                         PRBool aCompileEventHandlers) {
-    nsCOMPtr<nsIDocument> oldDoc = mDocument;
-
-    nsAutoString rel;
-    nsAutoString rev;
-    GetAttr(kNameSpaceID_None, nsHTMLAtoms::rel, rel);
-    GetAttr(kNameSpaceID_None, nsHTMLAtoms::rev, rev);
-    
-    CreateAndDispatchEvent(oldDoc, rel, rev, NS_LITERAL_STRING("DOMLinkRemoved"));
-
-    // Do the removal and addition into the new doc.
-    nsresult rv = nsGenericHTMLLeafElement::SetDocument(aDocument, aDeep,
-                                                        aCompileEventHandlers);
-    if (NS_SUCCEEDED(rv)) {
-      UpdateStyleSheet(oldDoc);
-    }
-		
-    CreateAndDispatchEvent(mDocument, rel, rev, NS_LITERAL_STRING("DOMLinkAdded"));
-
-    return rv;
-  }
- 
+  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                           PRBool aCompileEventHandlers);
   void CreateAndDispatchEvent(nsIDocument* aDoc, const nsString& aRel,
-                              const nsString& aRev, 
-                              const nsAString& aEventName) {
-    if (!aDoc)
-      return;
-
-    // In the unlikely case that both rev is specified *and* rel=stylesheet,
-    // this code will cause the event to fire, on the principle that maybe the
-    // page really does want to specify that it's author is a stylesheet. Since
-    // this should never actually happen and the performance hit is minimal,
-    // doing the "right" thing costs virtually nothing here, even if it doesn't
-    // make much sense.
-    if (aRev.IsEmpty() &&
-        (aRel.IsEmpty() || aRel.EqualsIgnoreCase("stylesheet")))
-      return;
-
-    nsCOMPtr<nsIDOMDocumentEvent> docEvent(do_QueryInterface(aDoc));
-    nsCOMPtr<nsIDOMEvent> event;
-    docEvent->CreateEvent(NS_LITERAL_STRING("Events"), getter_AddRefs(event));
-    if (event) {
-      event->InitEvent(aEventName, PR_TRUE, PR_TRUE);
-      PRBool noDefault;
-      nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(NS_STATIC_CAST(nsIDOMNode*, this)));
-      if (target)
-        target->DispatchEvent(event, &noDefault);
-    }
+                              const nsString& aRev,
+                              const nsAString& aEventName);
+  nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                   const nsAString& aValue, PRBool aNotify)
+  {
+    return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
   }
+  virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                           nsIAtom* aPrefix, const nsAString& aValue,
+                           PRBool aNotify);
+  virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                             PRBool aNotify);
 
-  NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAString& aValue, PRBool aNotify) {
-    nsresult rv = nsGenericHTMLLeafElement::SetAttr(aNameSpaceID, aName,
-                                                    aValue, aNotify);
-    if (NS_SUCCEEDED(rv)) {
-      UpdateStyleSheet();
-    }
-    return rv;
-  }
-  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
-                     PRBool aNotify) {
-    nsresult rv = nsGenericHTMLLeafElement::SetAttr(aNodeInfo, aValue,
-                                                    aNotify);
-
-    // nsGenericHTMLLeafElement::SetAttr(nsINodeInfo* aNodeInfo,
-    //                                   const nsAString& aValue,
-    //                                   PRBool aNotify)
-    //
-    // calls
-    //
-    // nsHTMLLinkElement::SetAttr(PRInt32 aNameSpaceID,
-    //                            nsIAtom* aName,
-    //                            const nsAString& aValue,
-    //                            PRBool aNotify)
-    //
-    // which ends up calling UpdateStyleSheet so we don't call UpdateStyleSheet
-    // here ourselves.
-
-    return rv;
-  }
-  NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
-                       PRBool aNotify) {
-    nsresult rv = nsGenericHTMLLeafElement::UnsetAttr(aNameSpaceID,
-                                                      aAttribute,
-                                                      aNotify);
-    if (NS_SUCCEEDED(rv)) {
-      UpdateStyleSheet();
-    }
-    return rv;
-  }
-
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
-                            nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
-                            nsEventStatus* aEventStatus);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
+  virtual nsresult HandleDOMEvent(nsIPresContext* aPresContext,
+                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
+                                  PRUint32 aFlags,
+                                  nsEventStatus* aEventStatus);
 
 protected:
   virtual void GetStyleSheetURL(PRBool* aIsInline,
-                                nsAString& aUrl);
+                                nsIURI** aURI);
   virtual void GetStyleSheetInfo(nsAString& aTitle,
                                  nsAString& aType,
                                  nsAString& aMedia,
@@ -203,7 +123,7 @@ protected:
 
 nsresult
 NS_NewHTMLLinkElement(nsIHTMLContent** aInstancePtrResult,
-                      nsINodeInfo *aNodeInfo)
+                      nsINodeInfo *aNodeInfo, PRBool aFromParser)
 {
   NS_ENSURE_ARG_POINTER(aInstancePtrResult);
 
@@ -231,12 +151,10 @@ NS_NewHTMLLinkElement(nsIHTMLContent** aInstancePtrResult,
 nsHTMLLinkElement::nsHTMLLinkElement()
   : mLinkState(eLinkState_Unknown)
 {
-  nsHTMLUtils::AddRef(); // for GetHrefCString
 }
 
 nsHTMLLinkElement::~nsHTMLLinkElement()
 {
-  nsHTMLUtils::Release(); // for GetHrefCString
 }
 
 
@@ -245,10 +163,10 @@ NS_IMPL_RELEASE_INHERITED(nsHTMLLinkElement, nsGenericElement)
 
 
 // QueryInterface implementation for nsHTMLLinkElement
-NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLLinkElement,
-                                    nsGenericHTMLLeafElement)
+NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLLinkElement, nsGenericHTMLElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLLinkElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMLinkStyle)
+  NS_INTERFACE_MAP_ENTRY(nsILink)
   NS_INTERFACE_MAP_ENTRY(nsIStyleSheetLinkingElement)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLLinkElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
@@ -273,7 +191,7 @@ nsHTMLLinkElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
   if (NS_FAILED(rv))
     return rv;
 
-  CopyInnerTo(this, it, aDeep);
+  CopyInnerTo(it, aDeep);
 
   *aReturn = NS_STATIC_CAST(nsIDOMNode *, it);
 
@@ -313,6 +231,7 @@ nsHTMLLinkElement::SetDisabled(PRBool aDisabled)
 
 
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Charset, charset)
+NS_IMPL_URI_ATTR(nsHTMLLinkElement, Href, href)
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Hreflang, hreflang)
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Media, media)
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Rel, rel)
@@ -320,61 +239,104 @@ NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Rev, rev)
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Target, target)
 NS_IMPL_STRING_ATTR(nsHTMLLinkElement, Type, type)
 
-
-NS_IMETHODIMP
-nsHTMLLinkElement::GetHref(nsAString& aValue)
+void
+nsHTMLLinkElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                               PRBool aCompileEventHandlers)
 {
-  char *buf;
-  nsresult rv = GetHrefCString(buf);
-  if (NS_FAILED(rv)) return rv;
-  if (buf) {
-    aValue.Assign(NS_ConvertASCIItoUCS2(buf));
-    nsCRT::free(buf);
+  nsCOMPtr<nsIDocument> oldDoc = mDocument;
+
+  nsAutoString rel;
+  nsAutoString rev;
+  GetAttr(kNameSpaceID_None, nsHTMLAtoms::rel, rel);
+  GetAttr(kNameSpaceID_None, nsHTMLAtoms::rev, rev);
+    
+  CreateAndDispatchEvent(oldDoc, rel, rev,
+                         NS_LITERAL_STRING("DOMLinkRemoved"));
+
+  // Do the removal and addition into the new doc.
+  nsGenericHTMLElement::SetDocument(aDocument, aDeep, aCompileEventHandlers);
+  UpdateStyleSheet(oldDoc);
+		
+  CreateAndDispatchEvent(mDocument, rel, rev,
+                         NS_LITERAL_STRING("DOMLinkAdded"));
+}
+ 
+void
+nsHTMLLinkElement::CreateAndDispatchEvent(nsIDocument* aDoc,
+                                          const nsString& aRel,
+                                          const nsString& aRev, 
+                                          const nsAString& aEventName)
+{
+  if (!aDoc)
+    return;
+
+  // In the unlikely case that both rev is specified *and* rel=stylesheet,
+  // this code will cause the event to fire, on the principle that maybe the
+  // page really does want to specify that it's author is a stylesheet. Since
+  // this should never actually happen and the performance hit is minimal,
+  // doing the "right" thing costs virtually nothing here, even if it doesn't
+  // make much sense.
+  if (aRev.IsEmpty() &&
+      (aRel.IsEmpty() || aRel.EqualsIgnoreCase("stylesheet")))
+    return;
+
+  nsCOMPtr<nsIDOMDocumentEvent> docEvent(do_QueryInterface(aDoc));
+  nsCOMPtr<nsIDOMEvent> event;
+  docEvent->CreateEvent(NS_LITERAL_STRING("Events"), getter_AddRefs(event));
+  if (event) {
+    event->InitEvent(aEventName, PR_TRUE, PR_TRUE);
+    nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(event));
+    privateEvent->SetTrusted(PR_TRUE);
+    PRBool noDefault;
+    nsCOMPtr<nsIDOMEventTarget> target =
+      do_QueryInterface(NS_STATIC_CAST(nsIDOMNode*, this));
+    if (target)
+      target->DispatchEvent(event, &noDefault);
   }
-
-  // NS_IMPL_STRING_ATTR does nothing where we have (buf == null)
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLLinkElement::SetHref(const nsAString& aValue)
+nsresult
+nsHTMLLinkElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                           nsIAtom* aPrefix, const nsAString& aValue,
+                           PRBool aNotify)
 {
-  // Clobber our "cache", so we'll recompute it the next time
-  // somebody asks for it.
-  mLinkState = eLinkState_Unknown;
+  if (aName == nsHTMLAtoms::href && kNameSpaceID_None == aNameSpaceID) {
+    SetLinkState(eLinkState_Unknown);
+  }
 
-  nsresult rv = nsGenericHTMLLeafElement::SetAttr(kNameSpaceID_HTML,
-                                                  nsHTMLAtoms::href,
-                                                  aValue,
-                                                  PR_TRUE);
+  nsresult rv = nsGenericHTMLElement::SetAttr(aNameSpaceID, aName, aPrefix,
+                                              aValue, aNotify);
   if (NS_SUCCEEDED(rv)) {
     UpdateStyleSheet();
   }
+
   return rv;
 }
 
-NS_IMETHODIMP
+nsresult
+nsHTMLLinkElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                             PRBool aNotify)
+{
+  nsresult rv = nsGenericHTMLElement::UnsetAttr(aNameSpaceID, aAttribute,
+                                                aNotify);
+  if (NS_SUCCEEDED(rv)) {
+    UpdateStyleSheet();
+  }
+
+  return rv;
+}
+
+nsresult
 nsHTMLLinkElement::HandleDOMEvent(nsIPresContext* aPresContext,
                            nsEvent* aEvent,
                            nsIDOMEvent** aDOMEvent,
                            PRUint32 aFlags,
                            nsEventStatus* aEventStatus)
 {
-  return HandleDOMEventForAnchors(this, aPresContext, aEvent, aDOMEvent,
+  return HandleDOMEventForAnchors(aPresContext, aEvent, aDOMEvent,
                                   aFlags, aEventStatus);
 }
 
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLLinkElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
 NS_IMETHODIMP
 nsHTMLLinkElement::GetLinkState(nsLinkState &aState)
 {
@@ -390,48 +352,17 @@ nsHTMLLinkElement::SetLinkState(nsLinkState aState)
 }
 
 NS_IMETHODIMP
-nsHTMLLinkElement::GetHrefCString(char* &aBuf)
+nsHTMLLinkElement::GetHrefURI(nsIURI** aURI)
 {
-  // Get href= attribute (relative URL).
-  nsAutoString relURLSpec;
-
-  if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLLeafElement::GetAttr(kNameSpaceID_None,
-                                        nsHTMLAtoms::href, relURLSpec)) {
-    // Clean up any leading or trailing whitespace
-    relURLSpec.Trim(" \t\n\r");
-
-    // Get base URL.
-    nsCOMPtr<nsIURI> baseURL;
-    GetBaseURL(*getter_AddRefs(baseURL));
-
-    if (baseURL) {
-      // Get absolute URL.
-      nsCAutoString buf;
-      NS_MakeAbsoluteURIWithCharset(buf, relURLSpec, mDocument, baseURL,
-                                    nsHTMLUtils::IOService,
-                                    nsHTMLUtils::CharsetMgr);
-      aBuf = ToNewCString(buf);
-    }
-    else {
-      // Absolute URL is same as relative URL.
-      aBuf = ToNewUTF8String(relURLSpec);
-    }
-  }
-  else {
-    // Absolute URL is empty because we have no HREF.
-    aBuf = nsnull;
-  }
-
-  return NS_OK;
+  return GetHrefURIForAnchors(aURI);
 }
 
 void
 nsHTMLLinkElement::GetStyleSheetURL(PRBool* aIsInline,
-                                    nsAString& aUrl)
+                                    nsIURI** aURI)
 {
   *aIsInline = PR_FALSE;
-  GetHref(aUrl);
+  GetHrefURIForAnchors(aURI);
   return;
 }
 
