@@ -45,6 +45,8 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIStreamListener.h"
+#include "nsILDAPConnection.h"
+#include "nsILDAPOperation.h"
 #include "nsILDAPMessageListener.h"
 #include "nsIProgressEventSink.h"
 #include "nsILDAPURL.h"
