@@ -773,6 +773,13 @@ nsXMLHttpRequest::CreateEvent(nsEvent* aEvent, nsIDOMEvent** aDOMEvent)
   privevent->SetCurrentTarget(this);
   privevent->SetOriginalTarget(this);
 
+  // We're not marking these events trusted as there is no way for us
+  // to tell if we're called through the code that normally calls us
+  // (i.e. necko) or if script on a webpage QI'd this object to say,
+  // nsIProgressEventSink and called onProgress(...).
+  //
+  // privevent->SetTrusted(?);
+
   return NS_OK;
 }
 
@@ -1343,7 +1350,7 @@ nsXMLHttpRequest::RequestCompleted()
   }
 
   // We need to create the event before nulling out mDocument
-  nsEvent evt(NS_PAGE_LOAD);
+  nsEvent evt(PR_TRUE, NS_PAGE_LOAD);
   nsCOMPtr<nsIDOMEvent> domevent;
   rv = CreateEvent(&evt, getter_AddRefs(domevent));
 
@@ -1567,8 +1574,31 @@ nsXMLHttpRequest::SetRequestHeader(const nsACString& header,
   if (!mChannel)             // open() initializes mChannel, and open()
     return NS_ERROR_FAILURE; // must be called before first setRequestHeader()
 
-  if (!IsASCII(header) || !IsASCII(value)) {
-    return NS_ERROR_INVALID_ARG;
+  // Prevent modification to certain HTTP headers (see bug 302263), unless
+  // the executing script has UniversalBrowserWrite permission.
+
+  nsCOMPtr<nsIScriptSecurityManager> secMan = 
+      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID);
+  if (!secMan)
+    return NS_ERROR_FAILURE;
+
+  PRBool privileged;
+  nsresult rv = secMan->IsCapabilityEnabled("UniversalBrowserWrite",
+                                            &privileged);
+  if (NS_FAILED(rv))
+    return NS_ERROR_FAILURE;
+
+  if (!privileged) {
+    const char *kInvalidHeaders[] = {
+      "host", "content-length", "transfer-encoding", "via", "upgrade"
+    };
+    for (size_t i = 0; i < NS_ARRAY_LENGTH(kInvalidHeaders); ++i) {
+      if (header.Equals(kInvalidHeaders[i],
+                        nsCaseInsensitiveCStringComparator())) {
+        NS_WARNING("refusing to set request header");
+        return NS_OK;
+      }
+    }
   }
 
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
@@ -1701,7 +1731,7 @@ nsXMLHttpRequest::Error(nsIDOMEvent* aEvent)
   // We need to create the event before nulling out mDocument
   nsCOMPtr<nsIDOMEvent> event(do_QueryInterface(aEvent));
   // There is no NS_PAGE_ERROR event but NS_SCRIPT_ERROR should be ok.
-  nsEvent evt(NS_SCRIPT_ERROR);
+  nsEvent evt(PR_TRUE, NS_SCRIPT_ERROR);
   if (!event) {
     CreateEvent(&evt, getter_AddRefs(event));
   }
@@ -1811,7 +1841,7 @@ nsXMLHttpRequest::OnProgress(nsIRequest *aRequest, nsISupports *aContext, PRUint
   if (mOnProgressListener)
   {
     nsCOMPtr<nsIDOMEvent> event;
-    nsEvent evt(NS_EVENT_NULL ); // what name do we make up here? 
+    nsEvent evt(PR_TRUE, NS_EVENT_NULL); // what name do we make up here? 
     nsresult rv = CreateEvent(&evt, getter_AddRefs(event));
     NS_ENSURE_SUCCESS(rv, rv);
     
