@@ -39,12 +39,12 @@
 
 // Defining this will trace the allocation of images.  This includes
 // ctor, dtor and update.
-//#define TRACE_IMAGE_ALLOCATION
+// #define TRACE_IMAGE_ALLOCATION
 
 //#define CHEAP_PERFORMANCE_MEASURMENT 1
 
 // Define this to see tiling debug output
-//#define DEBUG_TILING
+// #define DEBUG_TILING
 
 /* XXX we are simply creating a GC and setting its function to Copy.
    we shouldn't be doing this every time this method is called.  this creates
@@ -59,27 +59,27 @@ NS_IMPL_ISUPPORTS1(nsImageGTK, nsIImage)
 //------------------------------------------------------------
 
 nsImageGTK::nsImageGTK()
+  : mImageBits(nsnull)
+  , mImagePixmap(nsnull)
+  , mTrueAlphaBits(nsnull)
+  , mAlphaBits(nsnull)
+  , mAlphaPixmap(nsnull)
+  , mAlphaXImage(nsnull)
+  , mWidth(0)
+  , mHeight(0)
+  , mRowBytes(0)
+  , mSizeImage(0)
+  , mDecodedX1(PR_INT32_MAX)
+  , mDecodedY1(PR_INT32_MAX)
+  , mDecodedX2(0)
+  , mDecodedY2(0)
+  , mAlphaDepth(0)
+  , mTrueAlphaDepth(0)
+  , mIsSpacer(PR_TRUE)
+  , mPendingUpdate(PR_FALSE)
+  , mDepth(0)
+  , mOptimized(PR_FALSE)
 {
-  NS_INIT_REFCNT();
-  mImageBits = nsnull;
-  mWidth = 0;
-  mHeight = 0;
-  mDepth = 0;
-  mAlphaBits = mTrueAlphaBits = nsnull;
-  mAlphaPixmap = nsnull;
-  mImagePixmap = nsnull;
-  mAlphaXImage = nsnull;
-  mAlphaDepth = mTrueAlphaDepth = 0;
-  mRowBytes = 0;
-  mSizeImage = 0;
-  mAlphaHeight = 0;
-  mAlphaWidth = 0;
-  mNaturalWidth = 0;
-  mNaturalHeight = 0;
-  mIsSpacer = PR_TRUE;
-  mPendingUpdate = PR_FALSE;
-  mOptimized = PR_FALSE;
-
 #ifdef TRACE_IMAGE_ALLOCATION
   printf("nsImageGTK::nsImageGTK(this=%p)\n",
          this);
@@ -91,17 +91,17 @@ nsImageGTK::nsImageGTK()
 nsImageGTK::~nsImageGTK()
 {
   if(nsnull != mImageBits) {
-    delete[] mImageBits;
+    free(mImageBits);
     mImageBits = nsnull;
   }
 
   if (nsnull != mAlphaBits) {
-    delete[] mAlphaBits;
+    free(mAlphaBits);
     mAlphaBits = nsnull;
   }
 
   if (nsnull != mTrueAlphaBits) {
-    delete[] mTrueAlphaBits;
+    free(mTrueAlphaBits);
     mTrueAlphaBits = nsnull;
   }
 
@@ -124,50 +124,31 @@ nsImageGTK::~nsImageGTK()
 #endif
 }
 
+/* static */ void
+nsImageGTK::Shutdown()
+{
+  if (s1bitGC) {
+    gdk_gc_unref(s1bitGC);
+    s1bitGC = nsnull;
+  }
+  if (sXbitGC) {
+    gdk_gc_unref(sXbitGC);
+    sXbitGC = nsnull;
+  }
+}
+
 //------------------------------------------------------------
 
 nsresult nsImageGTK::Init(PRInt32 aWidth, PRInt32 aHeight,
                           PRInt32 aDepth, nsMaskRequirements aMaskRequirements)
 {
+  // Assumed: Init only gets called once by gfxIImageFrame
   g_return_val_if_fail ((aWidth != 0) || (aHeight != 0), NS_ERROR_FAILURE);
 
-  if (nsnull != mImageBits) {
-   delete[] mImageBits;
-   mImageBits = nsnull;
-  }
-
-  if (nsnull != mAlphaBits) {
-    delete[] mAlphaBits;
-    mAlphaBits = nsnull;
-  }
-
-  if (nsnull != mTrueAlphaBits) {
-    delete[] mTrueAlphaBits;
-    mTrueAlphaBits = nsnull;
-  }
-
-  if (nsnull != mAlphaPixmap) {
-    gdk_pixmap_unref(mAlphaPixmap);
-    mAlphaPixmap = nsnull;
-  }
-
-  if (nsnull != mAlphaXImage) {
-    mAlphaXImage->data = 0;
-    XDestroyImage(mAlphaXImage);
-    mAlphaXImage = nsnull;
-  }
-
-  SetDecodedRect(0,0,0,0);  //init
-  SetNaturalWidth(0);
-  SetNaturalHeight(0);
-
-  // mImagePixmap gets created once per unique image bits in Draw()
-  // ImageUpdated(nsImageUpdateFlags_kBitsChanged) can cause the
-  // image bits to change and mImagePixmap will be unrefed and nulled.
-  if (nsnull != mImagePixmap) {
-    gdk_pixmap_unref(mImagePixmap);
-    mImagePixmap = nsnull;
-  }
+  // X Protocol limits us to image dimensions less than 32767
+  // unless we want to go through lots of pain and suffering.
+  if (aWidth > SHRT_MAX || aHeight > SHRT_MAX)
+    return NS_ERROR_FAILURE;
 
   if (24 == aDepth) {
     mNumBytesPixel = 3;
@@ -179,7 +160,6 @@ nsresult nsImageGTK::Init(PRInt32 aWidth, PRInt32 aHeight,
   mWidth = aWidth;
   mHeight = aHeight;
   mDepth = aDepth;
-  mIsTopToBottom = PR_TRUE;
 
 #ifdef TRACE_IMAGE_ALLOCATION
   printf("nsImageGTK::Init(this=%p,%d,%d,%d,%d)\n",
@@ -193,24 +173,21 @@ nsresult nsImageGTK::Init(PRInt32 aWidth, PRInt32 aHeight,
   // create the memory for the image
   ComputeMetrics();
 
-  mImageBits = (PRUint8*) new PRUint8[mSizeImage];
+  mImageBits = (PRUint8*)malloc(mSizeImage);
+  if (!mImageBits)
+    return NS_ERROR_OUT_OF_MEMORY;
 
   switch(aMaskRequirements)
   {
-    case nsMaskRequirements_kNoMask:
-      mAlphaBits = nsnull;
-      mAlphaWidth = 0;
-      mAlphaHeight = 0;
-      break;
-
     case nsMaskRequirements_kNeeds8Bit:
       mTrueAlphaRowBytes = aWidth;
       mTrueAlphaDepth = 8;
 
       // 32-bit align each row
       mTrueAlphaRowBytes = (mTrueAlphaRowBytes + 3) & ~0x3;
-      mTrueAlphaBits = new PRUint8[mTrueAlphaRowBytes * aHeight];
-      memset(mTrueAlphaBits, 0, mTrueAlphaRowBytes*aHeight);
+      mTrueAlphaBits = (PRUint8*)calloc(mTrueAlphaRowBytes * aHeight, 1);
+      if (!mTrueAlphaBits)
+        return NS_ERROR_OUT_OF_MEMORY;
 
       // FALL THROUGH
 
@@ -221,11 +198,13 @@ nsresult nsImageGTK::Init(PRInt32 aWidth, PRInt32 aHeight,
       // 32-bit align each row
       mAlphaRowBytes = (mAlphaRowBytes + 3) & ~0x3;
 
-      mAlphaBits = new PRUint8[mAlphaRowBytes * aHeight];
-      memset(mAlphaBits, 0, mAlphaRowBytes*aHeight);
-      mAlphaWidth = aWidth;
-      mAlphaHeight = aHeight;
+      mAlphaBits = (PRUint8*)calloc(mAlphaRowBytes * aHeight, 1);
+      if (!mAlphaBits)
+        return NS_ERROR_OUT_OF_MEMORY;
       break;
+
+    default:
+      break; // avoid compiler warning
   }
 
   if (aMaskRequirements == nsMaskRequirements_kNeeds8Bit)
@@ -266,11 +245,6 @@ nsColorMap *nsImageGTK::GetColorMap()
   return nsnull;
 }
 
-PRBool nsImageGTK::IsOptimized()
-{
-  return PR_TRUE;
-}
-
 PRUint8 *nsImageGTK::GetAlphaBits()
 {
   if (mTrueAlphaBits)
@@ -279,16 +253,6 @@ PRUint8 *nsImageGTK::GetAlphaBits()
     return mAlphaBits;
 }
 
-PRInt32 nsImageGTK::GetAlphaWidth()
-{
-  return mAlphaWidth;
-}
-
-PRInt32 nsImageGTK::GetAlphaHeight()
-{
-  return mAlphaHeight;
-}
-
 PRInt32
 nsImageGTK::GetAlphaLineStride()
 {
@@ -298,38 +262,27 @@ nsImageGTK::GetAlphaLineStride()
     return mAlphaRowBytes;
 }
 
-nsIImage *nsImageGTK::DuplicateImage()
-{
-  return nsnull;
-}
-
-void nsImageGTK::SetAlphaLevel(PRInt32 aAlphaLevel)
-{
-}
-
-PRInt32 nsImageGTK::GetAlphaLevel()
-{
-  return 0;
-}
-
-void nsImageGTK::MoveAlphaMask(PRInt32 aX, PRInt32 aY)
-{
-}
-
 void nsImageGTK::ImageUpdated(nsIDeviceContext *aContext,
                               PRUint8 aFlags,
                               nsRect *aUpdateRect)
 {
   mPendingUpdate = PR_TRUE;
   mUpdateRegion.Or(mUpdateRegion, *aUpdateRect);
+
+  mDecodedX1 = PR_MIN(mDecodedX1, aUpdateRect->x);
+  mDecodedY1 = PR_MIN(mDecodedY1, aUpdateRect->y);
+
+  if (aUpdateRect->YMost() > mDecodedY2)
+    mDecodedY2 = aUpdateRect->YMost();
+  if (aUpdateRect->XMost() > mDecodedX2)
+    mDecodedX2 = aUpdateRect->XMost();
 }
 
 void nsImageGTK::UpdateCachedImage()
 {
 #ifdef TRACE_IMAGE_ALLOCATION
-  printf("nsImageGTK::ImageUpdated(this=%p,%d)\n",
-         this,
-         aFlags);
+  printf("nsImageGTK::ImageUpdated(this=%p)\n",
+         this);
 #endif
 
   nsRegionRectIterator ri(mUpdateRegion);
@@ -359,8 +312,20 @@ void nsImageGTK::UpdateCachedImage()
             break;
           case 0:
             NS_CLEAR_BIT(mask,x);
-            if (mAlphaDepth != 8)
+            if (mAlphaDepth == 0) {
               mAlphaDepth=1;
+
+              // promoting an image from no alpha channel to 1-bit, so
+              // we need to create/clear the alpha pixmap
+              CreateOffscreenPixmap(mWidth, mHeight);
+
+              XFillRectangle(GDK_WINDOW_XDISPLAY(mAlphaPixmap),
+                             GDK_WINDOW_XWINDOW(mAlphaPixmap),
+                             GDK_GC_XGC(s1bitGC),
+                             mDecodedX1, mDecodedY1,
+                             mDecodedX2 - mDecodedX1 + 1,
+                             mDecodedY2 - mDecodedY1 + 1);
+            }
             break;
           default:
             mAlphaDepth=8;
@@ -379,7 +344,7 @@ void nsImageGTK::UpdateCachedImage()
           mAlphaPixmap = 0;
         }
         if (mAlphaBits) {
-          delete [] mAlphaBits;
+          free(mAlphaBits);
           mAlphaBits = mTrueAlphaBits;
           mAlphaRowBytes = mTrueAlphaRowBytes;
           mTrueAlphaBits = 0;
@@ -434,7 +399,7 @@ void nsImageGTK::UpdateCachedImage()
       if (mIsSpacer && (leftindex <= rightindex)) {
         for (unsigned y=rect->y; (y<bottom) && mIsSpacer; y++) {
           unsigned char *alpha = mAlphaBits + mAlphaRowBytes*y + leftindex;
-          for (unsigned x=left; x<right; x++) {
+          for (unsigned x=leftindex; x<=rightindex; x++) {
             if (*(alpha++)!=0) {
               mIsSpacer = PR_FALSE;
               break;
@@ -467,7 +432,7 @@ void nsImageGTK::UpdateCachedImage()
     }
   }
   
-  mUpdateRegion.Empty();
+  mUpdateRegion.SetEmpty();
   mPendingUpdate = PR_FALSE;
   mFlags = nsImageUpdateFlags_kBitsChanged; // this should be 0'd out by Draw()
 }
@@ -521,8 +486,8 @@ XlibRectStretch(PRInt32 srcWidth, PRInt32 srcHeight,
   xd2 = dstWidth-1;
   yd2 = dstHeight-1;
 
-//  fprintf(stderr, "%p (%ld %ld)-(%ld %ld) (%ld %ld)-(%ld %ld)\n",
-//          aDstImage, xs1, ys1, xs2, ys2, xd1, yd1, xd2, yd2);
+//  fprintf(stderr, "XRS %p (%ld %ld)-(%ld %ld) (%ld %ld)-(%ld %ld)\n",
+//          (void *)aDstImage, xs1, ys1, xs2, ys2, xd1, yd1, xd2, yd2);
   
   startColumn = aDX-dstOrigX;
   startRow    = aDY-dstOrigY;
@@ -560,12 +525,18 @@ XlibRectStretch(PRInt32 srcWidth, PRInt32 srcHeight,
 
 //  fprintf(stderr, "scaleY Start/End = %d %d\n", scaleStartY, scaleEndY);
 
-  if (!skipHorizontal && !skipVertical)
+  if (!skipHorizontal && !skipVertical) {
     aTmpImage = gdk_pixmap_new(nsnull,
                                endColumn-startColumn,
                                scaleEndY-scaleStartY,
                                aDepth);
-  
+#ifdef MOZ_WIDGET_GTK2
+    if (aDepth != 1)
+      gdk_drawable_set_colormap(GDK_DRAWABLE(aTmpImage),
+                                gdk_rgb_get_colormap());
+#endif
+  }
+ 
   dx = abs((int)(yd2-yd1));
   dy = abs((int)(ys2-ys1));
   sx = sign(yd2-yd1);
@@ -663,6 +634,9 @@ nsImageGTK::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   if ((mAlphaDepth==1) && mIsSpacer)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
 #ifdef TRACE_IMAGE_ALLOCATION
   fprintf(stderr, "nsImageGTK::Draw(%p) s=(%4d %4d %4d %4d) d=(%4d %4d %4d %4d)\n",
          this,
@@ -752,6 +726,7 @@ nsImageGTK::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   if ((srcWidth != dstWidth) || (srcHeight != dstHeight)) {
     GdkPixmap *pixmap = 0;
     GdkGC *gc = 0;
+    nsRegionGTK clipRgn;
 
     switch (mAlphaDepth) {
     case 8:
@@ -778,14 +753,84 @@ nsImageGTK::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
           gdk_gc_set_clip_mask(gc, pixmap);
         }
       }
-      /* fall through */
+
+      if (gdk_rgb_get_visual()->depth <= 8) {
+        PRUint8 *scaledRGB = (PRUint8 *)nsMemory::Alloc(3*dstWidth*dstHeight);
+
+        if (!scaledRGB)
+          return NS_ERROR_OUT_OF_MEMORY;
+
+        RectStretch(mWidth, mHeight,
+                    dstWidth, dstHeight,
+                    0, 0, dstWidth-1, dstHeight-1,
+                    mImageBits, mRowBytes, scaledRGB, 3*dstWidth, 24);
+
+        if (NS_SUCCEEDED(((nsRenderingContextGTK&)aContext).CopyClipRegion(clipRgn))) {
+          // we have both a set of rectangles and a bitmap defining the clip
+          // let X11 clip to the bitmap, do the rectangles by hand
+          nsRegionRectSet *rectSet = nsnull;
+          clipRgn.Intersect(aDX, aDY, aDWidth, aDHeight);
+          clipRgn.GetRects(&rectSet);
+          for (PRUint32 i=0; i<rectSet->mRectsLen; i++) {
+            nsRegionRect *rect = &(rectSet->mRects[i]);
+
+            gdk_draw_rgb_image_dithalign(drawing->GetDrawable(), gc,
+                                         rect->x, rect->y, rect->width, rect->height,
+                                         GDK_RGB_DITHER_MAX, 
+                                         scaledRGB + 3*((rect->y-dstOrigY)*dstWidth+(rect->x-dstOrigX)),
+                                         3*dstWidth,
+                                         (rect->x-dstOrigX), (rect->y-dstOrigY));
+          }
+          clipRgn.FreeRects(rectSet);
+        } else {
+          gdk_draw_rgb_image_dithalign(drawing->GetDrawable(), gc,
+                                       aDX, aDY, aDWidth, aDHeight,
+                                       GDK_RGB_DITHER_MAX, 
+                                       scaledRGB + 3*((aDY-dstOrigY)*dstWidth+(aDX-dstOrigX)),
+                                       3*dstWidth,
+                                       (aDX-dstOrigX), (aDY-dstOrigY));
+        }
+        nsMemory::Free(scaledRGB);
+      } else {
+        if (NS_SUCCEEDED(((nsRenderingContextGTK&)aContext).CopyClipRegion(clipRgn))) {
+          // we have both a set of rectangles and a bitmap defining the clip
+          // let X11 clip to the bitmap, do the rectangles by hand
+          nsRegionRectSet *rectSet = nsnull;
+          clipRgn.Intersect(aDX, aDY, aDWidth, aDHeight);
+          clipRgn.GetRects(&rectSet);
+          for (PRUint32 i=0; i<rectSet->mRectsLen; i++) {
+            nsRegionRect *rect = &(rectSet->mRects[i]);
+            
+            XlibRectStretch(srcWidth, srcHeight,
+                            dstWidth, dstHeight,
+                            dstOrigX, dstOrigY,
+                            rect->x, rect->y,
+                            rect->width, rect->height,
+                            mImagePixmap, drawing->GetDrawable(),
+                            gc, sXbitGC, gdk_rgb_get_visual()->depth);
+          }
+          clipRgn.FreeRects(rectSet);
+        } else {
+          // only a mask
+          XlibRectStretch(srcWidth, srcHeight,
+                          dstWidth, dstHeight,
+                          dstOrigX, dstOrigY,
+                          aDX, aDY,
+                          aDWidth, aDHeight,
+                          mImagePixmap, drawing->GetDrawable(),
+                          gc, sXbitGC, gdk_rgb_get_visual()->depth);
+        }
+      }
+
+      break;
     case 0:
       if (!gc)
         gc = ((nsRenderingContextGTK&)aContext).GetGC();
 
       if (gdk_rgb_get_visual()->depth <= 8) {
         PRUint8 *scaledRGB = (PRUint8 *)nsMemory::Alloc(3*dstWidth*dstHeight);
-        RectStretch(0, 0, mWidth-1, mHeight-1,
+        RectStretch(mWidth, mHeight,
+                    dstWidth, dstHeight,
                     0, 0, dstWidth-1, dstHeight-1,
                     mImageBits, mRowBytes, scaledRGB, 3*dstWidth, 24);
     
@@ -842,15 +887,39 @@ nsImageGTK::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
     copyGC = ((nsRenderingContextGTK&)aContext).GetGC();
   }
 
-  gdk_window_copy_area(drawing->GetDrawable(),      // dest window
-                       copyGC,                      // gc
-                       aDX,                         // xdest
-                       aDY,                         // ydest
-                       mImagePixmap,                // source window
-                       aSX,                         // xsrc
-                       aSY,                         // ysrc
-                       aSWidth,                     // width
-                       aSHeight);                   // height
+  nsRegionGTK clipRgn;
+  if (mAlphaPixmap &&
+      NS_SUCCEEDED(((nsRenderingContextGTK&)aContext).CopyClipRegion(clipRgn))) {
+    // we have both a set of rectangles and a bitmap defining the clip
+    // let X11 clip to the bitmap, do the rectangles by hand
+    nsRegionRectSet *rectSet = nsnull;
+    clipRgn.Intersect(aDX, aDY, aSWidth, aSHeight);
+    clipRgn.GetRects(&rectSet);
+    for (PRUint32 i=0; i<rectSet->mRectsLen; i++) {
+      nsRegionRect *rect = &(rectSet->mRects[i]);
+      gdk_window_copy_area(drawing->GetDrawable(),      // dest window
+                           copyGC,                      // gc
+                           rect->x,                     // xdest
+                           rect->y,                     // ydest
+                           mImagePixmap,                // source window
+                           aSX+(rect->x-aDX),           // xsrc
+                           aSY+(rect->y-aDY),           // ysrc
+                           rect->width,                 // width
+                           rect->height);               // height
+    }
+    clipRgn.FreeRects(rectSet);
+  } else {
+    // normal case - let X11 take care of all the clipping
+    gdk_window_copy_area(drawing->GetDrawable(),      // dest window
+                         copyGC,                      // gc
+                         aDX,                         // xdest
+                         aDY,                         // ydest
+                         mImagePixmap,                // source window
+                         aSX,                         // xsrc
+                         aSY,                         // ysrc
+                         aSWidth,                     // width
+                         aSHeight);                   // height
+  }
  
   gdk_gc_unref(copyGC);
   mFlags = 0;
@@ -1049,8 +1118,6 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
   GdkVisual *visual     = gdk_rgb_get_visual();
   GdkColormap *colormap = gdk_rgb_get_cmap();
 
-  unsigned char *target = readData;
-
   // flip bytes
   if (flipBytes && (ximage->bits_per_pixel>=16)) {
     for (int row=0; row<ximage->height; row++) {
@@ -1101,9 +1168,10 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
   greenFill = 0xff>>visual->green_prec;
   blueFill =  0xff>>visual->blue_prec;
 
-  for (int row=0; row<ximage->height; row++) {
+  for (unsigned row=0; row<height; row++) {
     unsigned char *ptr = srcData + row*ximage->bytes_per_line;
-    for (int col=0; col<ximage->width; col++) {
+    unsigned char *target = readData+3*row*ximage->width;
+    for (unsigned col=0; col<width; col++) {
       unsigned pix;
       switch (ximage->bits_per_pixel) {
       case 1:
@@ -1168,7 +1236,7 @@ nsImageGTK::DrawCompositedGeneral(PRBool isLSB, PRBool flipBytes,
 
   // now composite
   for (unsigned y=0; y<height; y++) {
-    unsigned char *targetRow = readData+3*y*width;
+    unsigned char *targetRow = readData+3*y*ximage->width;
     unsigned char *imageRow  = imageOrigin + y*imageStride;
     unsigned char *alphaRow  = alphaOrigin + y*alphaStride;
     
@@ -1206,10 +1274,10 @@ nsImageGTK::DrawComposited(nsIRenderingContext &aContext,
   readWidth = aDWidth;
   readHeight = aDHeight;
 
-  //  fprintf(stderr, "aX=%d aY=%d, aWidth=%u aHeight=%u\n", aX, aY, aWidth, aHeight);
-  //  fprintf(stderr, "surfaceWidth=%u surfaceHeight=%u\n", surfaceWidth, surfaceHeight);
-  //  fprintf(stderr, "readX=%u readY=%u readWidth=%u readHeight=%u destX=%u destY=%u\n\n",
-  //          readX, readY, readWidth, readHeight, destX, destY);
+//  fprintf(stderr, "dstOrigX=%d dstOrigY=%d, dstWidth=%u dstHeight=%u\n", dstOrigX, dstOrigY, dstWidth, dstHeight);
+//  fprintf(stderr, "srcWidth=%u srcHeight=%u\n", srcWidth, srcHeight);
+//  fprintf(stderr, "readX=%u readY=%u readWidth=%u readHeight=%u destX=%u destY=%u\n\n",
+//          readX, readY, readWidth, readHeight, destX, destY);
 
   XImage *ximage = XGetImage(dpy, drawable,
                              readX, readY, readWidth, readHeight, 
@@ -1244,11 +1312,15 @@ nsImageGTK::DrawComposited(nsIRenderingContext &aContext,
         nsMemory::Free(scaledAlpha);
       return;
     }
-    RectStretch(x1, y1, x2-1, y2-1,
-                0, 0, readWidth-1, readHeight-1,
+    RectStretch(srcWidth, srcHeight,
+                dstWidth, dstHeight,
+                destX, destY,
+                destX+aDWidth-1, destY+aDHeight-1,
                 mImageBits, mRowBytes, scaledImage, 3*readWidth, 24);
-    RectStretch(x1, y1, x2-1, y2-1,
-                0, 0, readWidth-1, readHeight-1,
+    RectStretch(srcWidth, srcHeight,
+                dstWidth, dstHeight,
+                destX, destY,
+                destX+aDWidth-1, destY+aDHeight-1,
                 mAlphaBits, mAlphaRowBytes, scaledAlpha, readWidth, 8);
     imageOrigin = scaledImage;
     imageStride = 3*readWidth;
@@ -1338,7 +1410,8 @@ nsImageGTK::DrawCompositeTile(nsIRenderingContext &aContext,
   drawing->GetDimensions(&surfaceWidth, &surfaceHeight);
   
   int readX, readY;
-  unsigned readWidth, readHeight, destX, destY;
+  unsigned readWidth, readHeight;
+  PRInt32 destX, destY;
 
   if ((aDY>=(int)surfaceHeight) || (aDX>=(int)surfaceWidth) ||
       (aDY+aDHeight<=0) || (aDX+aDWidth<=0)) {
@@ -1400,7 +1473,7 @@ nsImageGTK::DrawCompositeTile(nsIRenderingContext &aContext,
   if (destY==mHeight)
     destY = 0;
 
-  for (PRInt32 y=0; y<readHeight; y+=compY) {
+  for (unsigned y=0; y<readHeight; y+=compY) {
     if (y==0) {
       compY = PR_MIN(mHeight-destY, readHeight-y);
     } else {
@@ -1411,7 +1484,7 @@ nsImageGTK::DrawCompositeTile(nsIRenderingContext &aContext,
     compTarget = readData + 3*y*ximage->width;
     compSource = (unsigned char *)ximage->data + y*ximage->bytes_per_line;
 
-    for (PRInt32 x=0; x<readWidth; x+=compX) {
+    for (unsigned x=0; x<readWidth; x+=compX) {
       if (x==0) {
         compX = PR_MIN(mWidth-destX, readWidth-x);
         imageOrigin = mImageBits + destY*mRowBytes + 3*destX;
@@ -1486,6 +1559,9 @@ void nsImageGTK::CreateOffscreenPixmap(PRInt32 aWidth, PRInt32 aHeight)
     // Create an off screen pixmap to hold the image bits.
     mImagePixmap = gdk_pixmap_new(nsnull, aWidth, aHeight,
                                   gdk_rgb_get_visual()->depth);
+#ifdef MOZ_WIDGET_GTK2
+    gdk_drawable_set_colormap(GDK_DRAWABLE(mImagePixmap), gdk_rgb_get_colormap());
+#endif
   }
 
     // Ditto for the clipmask
@@ -1515,8 +1591,11 @@ void nsImageGTK::CreateOffscreenPixmap(PRInt32 aWidth, PRInt32 aHeight)
        low to high address in memory. */
     mAlphaXImage->byte_order = MSBFirst;
 
-    if (!s1bitGC)
+    if (!s1bitGC) {
+      GdkColor fg = { 1, 0, 0, 0 };
       s1bitGC = gdk_gc_new(mAlphaPixmap);
+      gdk_gc_set_foreground(s1bitGC, &fg);
+    }
   }
 
   if (!sXbitGC)
@@ -1602,6 +1681,7 @@ void nsImageGTK::TilePixmap(GdkPixmap *src, GdkPixmap *dest,
 NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
                                    nsDrawingSurface aSurface,
                                    PRInt32 aSXOffset, PRInt32 aSYOffset,
+                                   PRInt32 aPadX, PRInt32 aPadY,
                                    const nsRect &aTileRect)
 {
 #ifdef DEBUG_TILING
@@ -1610,16 +1690,15 @@ NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
          aTileRect.x, aTileRect.y,
          aTileRect.width, aTileRect.height, this);
 #endif
-
-  if (mPendingUpdate)
-    UpdateCachedImage();
-
   if (mPendingUpdate)
     UpdateCachedImage();
 
   if ((mAlphaDepth==1) && mIsSpacer)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   nsDrawingSurfaceGTK *drawing = (nsDrawingSurfaceGTK*)aSurface;
   PRBool partial = PR_FALSE;
 
@@ -1655,7 +1734,7 @@ NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
     return NS_OK;
   }
 
-  if (partial || (mAlphaDepth == 8)) {
+  if (partial || (mAlphaDepth == 8) || (aPadX || aPadY)) {
     PRInt32 aY0 = aTileRect.y - aSYOffset,
             aX0 = aTileRect.x - aSXOffset,
             aY1 = aTileRect.y + aTileRect.height,
@@ -1664,8 +1743,9 @@ NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
     // Set up clipping and call Draw().
     PRBool clipState;
     aContext.PushState();
-    aContext.SetClipRect(aTileRect, nsClipCombine_kIntersect,
-                         clipState);
+    ((nsRenderingContextGTK&)aContext).SetClipRectInPixels(
+      aTileRect, nsClipCombine_kIntersect, clipState);
+    ((nsRenderingContextGTK&)aContext).UpdateGC();
 
     if (mAlphaDepth==8) {
       DrawCompositeTile(aContext, aSurface,
@@ -1676,8 +1756,8 @@ NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
 #ifdef DEBUG_TILING
       printf("Warning: using slow tiling\n");
 #endif
-      for (PRInt32 y = aY0; y < aY1; y += mHeight)
-        for (PRInt32 x = aX0; x < aX1; x += mWidth)
+      for (PRInt32 y = aY0; y < aY1; y += mHeight + aPadY)
+        for (PRInt32 x = aX0; x < aX1; x += mWidth + aPadX)
           Draw(aContext,aSurface, x,y,
                PR_MIN(validWidth, aX1-x),
                PR_MIN(validHeight, aY1-y));
@@ -1696,6 +1776,9 @@ NS_IMETHODIMP nsImageGTK::DrawTile(nsIRenderingContext &aContext,
 
     tileImg = gdk_pixmap_new(nsnull, aTileRect.width, 
                              aTileRect.height, drawing->GetDepth());
+#ifdef MOZ_WIDGET_GTK2
+    gdk_drawable_set_colormap(GDK_DRAWABLE(tileImg), gdk_rgb_get_colormap());
+#endif
     TilePixmap(mImagePixmap, tileImg, aSXOffset, aSYOffset, tmpRect,
                tmpRect, PR_FALSE);
 
@@ -1746,18 +1829,18 @@ nsresult nsImageGTK::Optimize(nsIDeviceContext* aContext)
 
   if ((gdk_rgb_get_visual()->depth > 8) && (mAlphaDepth != 8)) {
     if(nsnull != mImageBits) {
-      delete[] mImageBits;
+      free(mImageBits);
       mImageBits = nsnull;
     }
 
     if (nsnull != mAlphaBits) {
-      delete[] mAlphaBits;
+      free(mAlphaBits);
       mAlphaBits = nsnull;
     }
   }
     
   if (mTrueAlphaBits) {
-    delete[] mTrueAlphaBits;
+    free(mTrueAlphaBits);
     mTrueAlphaBits = nsnull;
   }
 
@@ -1791,7 +1874,10 @@ nsImageGTK::LockImagePixels(PRBool aMaskPixels)
                         0, 0, mWidth, mHeight,
                         AllPlanes, XYPixmap);
 
-    mImageBits = (PRUint8*) new PRUint8[mSizeImage];
+    mImageBits = (PRUint8*)malloc(mSizeImage);
+    if (!mImageBits)
+      return NS_ERROR_OUT_OF_MEMORY;
+
     GdkVisual *visual = gdk_rgb_get_visual();
     GdkColormap *colormap = gdk_rgb_get_cmap();
 
@@ -1864,23 +1950,6 @@ nsImageGTK::UnlockImagePixels(PRBool aMaskPixels)
   return NS_OK;
 } 
 
-// ---------------------------------------------------
-//	Set the decoded dimens of the image
-//
-NS_IMETHODIMP
-nsImageGTK::SetDecodedRect(PRInt32 x1, PRInt32 y1, PRInt32 x2, PRInt32 y2 )
-{
-    
-  mDecodedX1 = x1; 
-  mDecodedY1 = y1; 
-  mDecodedX2 = x2; 
-  mDecodedY2 = y2; 
-
-
-  return NS_OK;
-}
-
-#ifdef USE_IMG2
 NS_IMETHODIMP nsImageGTK::DrawToImage(nsIImage* aDstImage,
                                       nscoord aDX, nscoord aDY,
                                       nscoord aDWidth, nscoord aDHeight)
@@ -1889,6 +1958,9 @@ NS_IMETHODIMP nsImageGTK::DrawToImage(nsIImage* aDstImage,
 
   if (!dest)
     return NS_ERROR_FAILURE;
+    
+  if (aDX >= dest->mWidth || aDY >= dest->mHeight)
+    return NS_OK;
 
   PRUint8 *rgbPtr=0, *alphaPtr=0;
   PRUint32 rgbStride, alphaStride;
@@ -1934,9 +2006,9 @@ NS_IMETHODIMP nsImageGTK::DrawToImage(nsIImage* aDstImage,
           else {
             dstAlpha[(aDX+x)>>3]       |= alphaPixels >> offset;
             // avoid write if no 1's to write - also avoids going past end of array
-            // compiler should merge the common sub-expressions
-            if (alphaPixels << (8U - offset))
-              dstAlpha[((aDX+x)>>3) + 1] |= alphaPixels << (8U - offset);
+            PRUint8 alphaTemp = alphaPixels << (8U - offset);
+            if (alphaTemp & 0xff)
+              dstAlpha[((aDX+x)>>3) + 1] |= alphaTemp;
           }
           
           if (alphaPixels == 0xff) {
@@ -1985,4 +2057,3 @@ NS_IMETHODIMP nsImageGTK::DrawToImage(nsIImage* aDstImage,
 
   return NS_OK;
 }
-#endif
