@@ -20,7 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * Contributor(s): rickg@netscape.com 
+ *          rickg@netscape.com 
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -38,11 +38,10 @@
  * ***** END LICENSE BLOCK ***** */
        
 //#define ENABLE_CRC              
-//#define RICKG_DEBUG           
-     
           
 #include "nsDebug.h"      
-#include "COtherDTD.h" 
+#include "nsIAtom.h"
+#include "COtherDTD.h"
 #include "nsHTMLTokens.h"
 #include "nsCRT.h"     
 #include "nsParser.h"  
@@ -54,10 +53,8 @@
 #include "prio.h"
 #include "plstr.h"  
 #include "nsDTDUtils.h"
-#include "nsTagHandler.h" 
 #include "nsHTMLTokenizer.h"
 #include "nsTime.h"
-#include "nsViewSourceHTML.h" 
 #include "nsParserNode.h"
 #include "nsHTMLEntities.h"
 #include "nsLinebreakConverter.h"
@@ -65,12 +62,10 @@
 
 #include "prmem.h" 
 
-
 static NS_DEFINE_IID(kIHTMLContentSinkIID, NS_IHTML_CONTENT_SINK_IID);
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);                 
 static NS_DEFINE_IID(kIDTDIID,      NS_IDTD_IID);
 static NS_DEFINE_IID(kClassIID,     NS_IOTHERHTML_DTD_IID); 
-  static NS_DEFINE_IID(kParserServiceCID, NS_PARSERSERVICE_CID);
 static const char kVerificationDir[] = "c:/temp";
  
 
@@ -145,7 +140,6 @@ NS_IMPL_RELEASE(COtherDTD)
  *  @return   
  */ 
 COtherDTD::COtherDTD() : nsIDTD() {
-  NS_INIT_REFCNT();
   mSink = 0; 
   mParser=0;        
   mLineNumber=1;  
@@ -176,14 +170,6 @@ COtherDTD::COtherDTD() : nsIDTD() {
   if(!gElementTable) {
     gElementTable = new CElementTable();
   }
-
-#ifdef  RICKG_DEBUG
-  //DebugDumpContainmentRules2(*this,"c:/temp/DTDRules.new","New COtherDTD Containment Rules");
-  nsHTMLElement::DebugDumpContainment("c:/temp/contain.new","ElementTable Rules");
-  nsHTMLElement::DebugDumpMembership("c:/temp/membership.out");
-  nsHTMLElement::DebugDumpContainType("c:/temp/ctnrules.out");
-#endif
-
 }
 
 /**
@@ -279,7 +265,8 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
       }
       else if(PR_TRUE==aParserContext.mMimeType.EqualsWithConversion(kHTMLTextContentType)) {
         switch(aParserContext.mDTDMode) {
-          case eDTDMode_strict:
+          case eDTDMode_full_standards:
+          case eDTDMode_almost_standards:
             result=ePrimaryDetect;
             break;
           default:
@@ -296,7 +283,8 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
             aParserContext.SetMimeType(NS_LITERAL_CSTRING(kHTMLTextContentType));
             if(!theBufHasXML) {
               switch(aParserContext.mDTDMode) {
-                case eDTDMode_strict:
+                case eDTDMode_full_standards:
+                case eDTDMode_almost_standards:
                   result=ePrimaryDetect;
                   break;
                 default:
@@ -309,7 +297,7 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
         }
       }
     } 
-  } 
+  }
   return result; 
 }
 
@@ -440,37 +428,40 @@ nsresult COtherDTD::DidBuildModel(nsresult anErrorCode,PRBool aNotifySink,nsIPar
 
           PRInt32 theIndex=mBodyContext->GetCount()-1;
           eHTMLTags theChild = mBodyContext->TagAt(theIndex); 
-          while(theIndex>0) {             
-            eHTMLTags theParent= mBodyContext->TagAt(--theIndex);
-            CElement *theElement=gElementTable->mElements[theParent];
-            nsCParserNode *theNode=mBodyContext->PeekNode();
+          while (theIndex>0) {             
+            eHTMLTags theParent = mBodyContext->TagAt(--theIndex);
+            CElement *theElement = gElementTable->mElements[theParent];
+            nsCParserNode *theNode = mBodyContext->PeekNode();
             theElement->HandleEndToken(theNode,theChild,mBodyContext,mSink);
-            theChild=theParent;
+            theChild = theParent;
           }
 
-          nsEntryStack *theChildStyles=0;
-          nsCParserNode* theNode=(nsCParserNode*)mBodyContext->Pop(theChildStyles);   
-          if(theNode) {
-            mSink->CloseHTML(*theNode);
+          nsEntryStack*  theChildStyles = 0;
+          nsCParserNode* theNode = (nsCParserNode*)mBodyContext->Pop(theChildStyles);   
+          if (theNode) {
+            mSink->CloseHTML();
           }
-
+          NS_ASSERTION(!theChildStyles, "there should no residual style information in this dtd");
+          IF_DELETE(theChildStyles, mNodeAllocator);
         }       
         else {
           //If you're here, then an error occured, but we still have nodes on the stack.
           //At a minimum, we should grab the nodes and recycle them.
           //Just to be correct, we'll also recycle the nodes. 
   
-          while(mBodyContext->GetCount() > 0) {  
+          while (mBodyContext->GetCount() > 0) {  
  
-            nsEntryStack *theChildStyles=0;
-            nsCParserNode* theNode=(nsCParserNode*)mBodyContext->Pop(theChildStyles);
-            if(theNode) {
-              theNode->mUseCount=0;
-              if(theChildStyles) {
+            nsEntryStack *theChildStyles = 0;
+            nsCParserNode* theNode = (nsCParserNode*)mBodyContext->Pop(theChildStyles);
+            if (theNode) {
+              theNode->mUseCount = 0;
+              if (theChildStyles) {
                 delete theChildStyles;
               } 
               IF_FREE(theNode, mNodeAllocator);
             }
+            NS_ASSERTION(!theChildStyles, "there should no residual style information in this dtd");
+            IF_DELETE(theChildStyles, mNodeAllocator);
           }    
         }    
   
@@ -478,7 +469,7 @@ nsresult COtherDTD::DidBuildModel(nsresult anErrorCode,PRBool aNotifySink,nsIPar
     } //if aparser  
 
       //No matter what, you need to call did build model.
-    result=aSink->DidBuildModel(0); 
+    result = aSink->DidBuildModel(); 
 
   } //if asink
   return result; 
@@ -559,7 +550,7 @@ nsresult COtherDTD::HandleToken(CToken* aToken,nsIParser* aParser){
  * This gets called after we've handled a given start tag.
  * It's a generic hook to let us to post processing.
  * @param   aToken contains the tag in question
- * @param   aChildTag is the tag itself.
+ * @param   aTag is the tag itself.
  * @return  status
  */
 nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
@@ -578,7 +569,7 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
         if(theNextToken)  {
           eHTMLTokenTypes theType=eHTMLTokenTypes(theNextToken->GetTokenType());
           if(eToken_newline==theType){
-            mLineNumber++;
+            ++mLineNumber;
             mTokenizer->PopToken();  //skip 1st newline inside PRE and LISTING
           }//if
         }//if
@@ -596,7 +587,7 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
 
         if(theCount) {
           PRInt32 theIndex=0;
-          for(theIndex=0;theIndex<theCount;theIndex++){
+          for(theIndex=0;theIndex<theCount;++theIndex){
             const nsAString& theKey = aNode.GetKeyAt(theIndex);
             if(theKey.Equals(NS_LITERAL_STRING("ENTITY"), nsCaseInsensitiveStringComparator())) {
               const nsAString& theName=aNode.GetValueAt(theIndex);
@@ -623,16 +614,6 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
   return result;
 } 
  
-
-#ifdef  RICKG_DEBUG
-void WriteTokenToLog(CToken* aToken) {
-
-  static nsFileSpec fileSpec("c:\\temp\\tokenlog.html");
-  static nsOutputFileStream outputStream(fileSpec);
-  aToken->DebugDumpSource(outputStream); //write token without close bracket...
-}
-#endif
- 
 /**
  * This gets called before we've handled a given start tag.
  * It's a generic hook to let us do pre processing.
@@ -659,7 +640,7 @@ nsresult COtherDTD::WillHandleStartTag(CToken* aToken,eHTMLTags aTag,nsIParserNo
 
     switch(aTag) {
       case eHTMLTag_newline:
-        mLineNumber++;
+        ++mLineNumber;
         break;
       default:
         break; 
@@ -689,9 +670,6 @@ nsresult COtherDTD::WillHandleStartTag(CToken* aToken,eHTMLTags aTag,nsIParserNo
  *  @return  PR_TRUE if all went well; PR_FALSE if error occured
  */      
 nsresult COtherDTD::HandleStartToken(CToken* aToken) {  
-  #ifdef  RICKG_DEBUG
-    WriteTokenToLog(aToken);
-  #endif
 
   //Begin by gathering up attributes...  
  
@@ -718,7 +696,7 @@ nsresult COtherDTD::HandleStartToken(CToken* aToken) {
           case eHTMLTag_html:  
             if(!mBodyContext->HasOpenContainer(theChildTag)){
               mSink->OpenHTML(*theNode);
-              mBodyContext->Push(theNode,0);
+              mBodyContext->Push(theNode, 0, PR_FALSE);
             } 
             theTagWasHandled=PR_TRUE;   
             break;         
@@ -762,10 +740,6 @@ nsresult COtherDTD::HandleEndToken(CToken* aToken) {
   nsresult    result=NS_OK;
   eHTMLTags   theChildTag=(eHTMLTags)aToken->GetTypeID();
  
-  #ifdef  RICKG_DEBUG    
-    WriteTokenToLog(aToken); 
-  #endif  
-  
   switch(theChildTag) {    
  
     case eHTMLTag_body: //we intentionally don't let the user close HTML or BODY
@@ -812,7 +786,7 @@ nsresult COtherDTD::CollectAttributes(nsIParserNode& aNode,eHTMLTags aTag,PRInt3
   if(aCount<=theAvailTokenCount) {
     //gElementTable->mElements[aTag]->GetSkipTarget();
     CToken* theToken=0; 
-    for(attr=0;attr<aCount;attr++){  
+    for(attr=0;attr<aCount;++attr){  
       theToken=mTokenizer->PopToken(); 
       if(theToken)  {
         // Sanitize the key for it might contain some non-alpha-non-digit characters
@@ -821,10 +795,6 @@ nsresult COtherDTD::CollectAttributes(nsIParserNode& aNode,eHTMLTags aTag,PRInt3
         // a legitimate "SELECTED" key.
         ((CAttributeToken*)theToken)->SanitizeKey();
  
-  #ifdef  RICKG_DEBUG
-    WriteTokenToLog(theToken);
-  #endif
- 
         aNode.AddAttribute(theToken); 
       }
     }
@@ -936,15 +906,17 @@ COtherDTD::IntTagToStringTag(PRInt32 aIntTag) const
   return str_ptr;
 }  
 
-NS_IMETHODIMP
-COtherDTD::ConvertEntityToUnicode(const nsAString& aEntity,
-                                  PRInt32* aUnicode) const
+NS_IMETHODIMP_(nsIAtom *)
+COtherDTD::IntTagToAtom(PRInt32 aIntTag) const
 {
-  *aUnicode = nsHTMLEntities::EntityToUnicode(aEntity);
+  nsIAtom *atom = nsHTMLTags::GetAtom((nsHTMLTag)aIntTag);
+
+  NS_ASSERTION(atom, "Bad tag enum passed to COtherDTD::IntTagToAtom()"
+               "!!");
+
+  return atom;
+}  
 
-  return NS_OK;
-}
- 
 /**
  *  This method is called to determine whether or not
  *  the given childtag is a block element.
