@@ -39,14 +39,13 @@
 #include "nsHTMLContainerFrame.h"
 #include "nsCSSRendering.h"
 #include "nsIDocument.h"
-#include "nsHTMLReflowCommand.h"
+#include "nsReflowPath.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsViewsCID.h"
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsIWidget.h"
-#include "nsHTMLIIDs.h"
 #include "nsPageFrame.h"
 #include "nsIRenderingContext.h"
 #include "nsGUIEvent.h"
@@ -96,8 +95,9 @@ public:
   NS_IMETHOD Init(nsIPresContext*  aPresContext,
               nsIContent*      aContent,
               nsIFrame*        aParent,
-              nsIStyleContext* aContext,
+              nsStyleContext*  aContext,
               nsIFrame*        aPrevInFlow);
+  NS_IMETHOD Destroy(nsIPresContext* aPresContext);
 
   NS_IMETHOD AppendFrames(nsIPresContext* aPresContext,
                           nsIPresShell&   aPresShell,
@@ -124,10 +124,7 @@ public:
                               const nsPoint& aPoint, 
                               nsFramePaintLayer aWhichLayer,
                               nsIFrame**     aFrame);
-  NS_IMETHOD IsPercentageBase(PRBool& aBase) const {
-    aBase = PR_TRUE;
-    return NS_OK;
-  }
+  virtual PRBool IsContainingBlock() const { return PR_TRUE; }
 
   NS_IMETHOD Paint(nsIPresContext*      aPresContext,
                    nsIRenderingContext& aRenderingContext,
@@ -147,18 +144,10 @@ public:
    *
    * @see nsLayoutAtoms::canvasFrame
    */
-  NS_IMETHOD GetFrameType(nsIAtom** aType) const;
+  virtual nsIAtom* GetType() const;
 
-  NS_IMETHOD AttributeChanged(nsIPresContext* aPresContext,
-                              nsIContent*     aChild,
-                              PRInt32         aNameSpaceID,
-                              nsIAtom*        aAttribute,
-                              PRInt32         aModType,
-                              PRInt32         aHint);
-  
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const;
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const;
 #endif
   NS_IMETHOD GetContentForEvent(nsIPresContext* aPresContext,
                                 nsEvent* aEvent,
@@ -169,8 +158,8 @@ protected:
 
   // Data members
   PRPackedBool             mDoPaintFocus;
-  nsCOMPtr<nsIPresContext> mPresContext;
-
+  nsCOMPtr<nsIViewManager> mViewManager;
+  nscoord                  mSavedChildWidth, mSavedChildHeight;
 
 private:
   NS_IMETHOD_(nsrefcnt) AddRef() { return NS_OK; }
@@ -223,27 +212,32 @@ NS_IMETHODIMP
 CanvasFrame::Init(nsIPresContext*  aPresContext,
               nsIContent*      aContent,
               nsIFrame*        aParent,
-              nsIStyleContext* aContext,
+              nsStyleContext*  aContext,
               nsIFrame*        aPrevInFlow)
 {
   nsresult rv = nsHTMLContainerFrame::Init(aPresContext,aContent,aParent,aContext,aPrevInFlow);
 
-  mPresContext = aPresContext;
-
-  nsCOMPtr<nsIPresShell> presShell;
-  aPresContext->GetShell(getter_AddRefs(presShell));
-  nsCOMPtr<nsIViewManager> vm;
-  presShell->GetViewManager(getter_AddRefs(vm));
+  mViewManager = aPresContext->GetViewManager();
 
   nsIScrollableView* scrollingView = nsnull;
-  vm->GetRootScrollableView(&scrollingView);
-
+  mViewManager->GetRootScrollableView(&scrollingView);
   if (scrollingView) {
-    scrollingView->AddScrollPositionListener((nsIScrollPositionListener *)this);
+    scrollingView->AddScrollPositionListener(this);
   }
 
   return rv;
+}
 
+NS_IMETHODIMP
+CanvasFrame::Destroy(nsIPresContext* aPresContext)
+{
+  nsIScrollableView* scrollingView = nsnull;
+  mViewManager->GetRootScrollableView(&scrollingView);
+  if (scrollingView) {
+    scrollingView->RemoveScrollPositionListener(this);
+  }
+
+  return nsHTMLContainerFrame::Destroy(aPresContext);
 }
 
 NS_IMETHODIMP
@@ -252,8 +246,13 @@ CanvasFrame::ScrollPositionWillChange(nsIScrollableView* aScrollable, nscoord aX
 #ifdef DEBUG_CANVAS_FOCUS
   {
     PRBool hasFocus = PR_FALSE;
+    nsCOMPtr<nsIViewObserver> observer;
+    mViewManager->GetViewObserver(*getter_AddRefs(observer));
+    nsCOMPtr<nsIPresShell> shell = do_QueryInterface(observer);
+    nsCOMPtr<nsIPresContext> context;
+    shell->GetPresContext(getter_AddRefs(context));
     nsCOMPtr<nsISupports> container;
-    mPresContext->GetContainer(getter_AddRefs(container));
+    context->GetContainer(getter_AddRefs(container));
     nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(container));
     if (docShell) {
       docShell->GetHasFocus(&hasFocus);
@@ -264,16 +263,7 @@ CanvasFrame::ScrollPositionWillChange(nsIScrollableView* aScrollable, nscoord aX
 
   if (mDoPaintFocus) {
     mDoPaintFocus = PR_FALSE;
-    
-    nsCOMPtr<nsIPresShell> presShell;
-    mPresContext->GetShell(getter_AddRefs(presShell));
-    if (presShell) {
-      nsCOMPtr<nsIViewManager> vm;
-      presShell->GetViewManager(getter_AddRefs(vm));
-      if (vm) {
-        vm->UpdateAllViews(NS_VMREFRESH_NO_SYNC);
-      }
-    }
+    mViewManager->UpdateAllViews(NS_VMREFRESH_NO_SYNC);
   }
   return NS_OK;
 }
@@ -357,9 +347,9 @@ CanvasFrame::RemoveFrame(nsIPresContext* aPresContext,
   } else if (aOldFrame == mFrames.FirstChild()) {
     // It's our one and only child frame
     // Damage the area occupied by the deleted frame
-    nsRect  damageRect;
-    aOldFrame->GetRect(damageRect);
-    Invalidate(aPresContext, damageRect, PR_FALSE);
+    // The child of the canvas probably can't have an outline, but why bother
+    // thinking about that?
+    Invalidate(aOldFrame->GetOutlineRect() + aOldFrame->GetPosition(), PR_FALSE);
 
     // Remove the frame and destroy it
     mFrames.DestroyFrame(aPresContext, aOldFrame);
@@ -385,17 +375,28 @@ CanvasFrame::Paint(nsIPresContext*      aPresContext,
                    nsFramePaintLayer    aWhichLayer,
                    PRUint32             aFlags)
 {
+  // We are wrapping the root frame of a document. We
+  // need to check the pres shell to find out if painting is locked
+  // down (because we're still in the early stages of document
+  // and frame construction.  If painting is locked down, then we
+  // do not paint our children.  
+  PRBool paintingSuppressed = PR_FALSE;
+  aPresContext->PresShell()->IsPaintingSuppressed(&paintingSuppressed);
+  if (paintingSuppressed) {
+    if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+      PaintSelf(aPresContext, aRenderingContext, aDirtyRect);
+    }
+    return NS_OK;
+  }
+
   nsresult rv = nsHTMLContainerFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 
   if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
 
 #ifdef DEBUG_CANVAS_FOCUS
     nsCOMPtr<nsIContent> focusContent;
-    nsCOMPtr<nsIEventStateManager> esm;
-    aPresContext->GetEventStateManager(getter_AddRefs(esm));
-    if (esm) {
-      esm->GetFocusedContent(getter_AddRefs(focusContent));
-    }
+    aPresContext->EventStateManager()->
+      GetFocusedContent(getter_AddRefs(focusContent));
 
     PRBool hasFocus = PR_FALSE;
     nsCOMPtr<nsISupports> container;
@@ -412,65 +413,51 @@ CanvasFrame::Paint(nsIPresContext*      aPresContext,
 #endif
 
     if (mDoPaintFocus) {
-      nsRect focusRect;
-      GetRect(focusRect);
+      nsRect focusRect = GetRect();
       /////////////////////
       // draw focus
       // XXX This is only temporary
       const nsStyleVisibility* vis = (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
       // Only paint the focus if we're visible
       if (vis->IsVisible()) {
-        nsCOMPtr<nsIEventStateManager> stateManager;
-        nsresult rv = aPresContext->GetEventStateManager(getter_AddRefs(stateManager));
-        if (NS_SUCCEEDED(rv)) {
-          nsIFrame * parentFrame;
-          GetParent(&parentFrame);
-          nsIScrollableFrame* scrollableFrame;
-          if (NS_SUCCEEDED(parentFrame->QueryInterface(NS_GET_IID(nsIScrollableFrame), (void**)&scrollableFrame))) {
-            nscoord width, height;
-            scrollableFrame->GetClipSize(aPresContext, &width, &height);
-          }
-          nsIView* parentView;
-          parentFrame->GetView(aPresContext, &parentView);
+        nsIFrame * parentFrame = GetParent();
+        nsIView* parentView = parentFrame->GetView();
 
-          nsIScrollableView* scrollableView;
-          if (NS_SUCCEEDED(parentView->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollableView))) {
-            nscoord width, height;
-            scrollableView->GetContainerSize(&width, &height);
-            const nsIView* clippedView;
-            scrollableView->GetClipView(&clippedView);
-            nsRect vcr;
-            clippedView->GetBounds(vcr);
-            focusRect.width = vcr.width;
-            focusRect.height = vcr.height;
-            nscoord x,y;
-            scrollableView->GetScrollPosition(x, y);
-            focusRect.x += x;
-            focusRect.y += y;
-          }
-
-          nsStyleOutline outlineStyle(aPresContext);
-          outlineStyle.SetOutlineStyle(NS_STYLE_BORDER_STYLE_DOTTED);
-          outlineStyle.SetOutlineInvert();
-          
-          float p2t;
-          aPresContext->GetPixelsToTwips(&p2t);
-          // XXX the CSS border for links is specified as 2px, but it
-          // is only drawn as 1px.  Match this here.
-          nscoord onePixel = NSIntPixelsToTwips(1, p2t);
-
-          nsRect borderInside(focusRect.x + onePixel,
-                              focusRect.y + onePixel,
-                              focusRect.width - 2 * onePixel,
-                              focusRect.height - 2 * onePixel);
-
-          nsCSSRendering::DrawDashedSides(0, aRenderingContext, 
-                                          focusRect, nsnull,
-                                          nsnull, &outlineStyle,
-                                          PR_TRUE, focusRect,
-                                          borderInside, 0, 
-                                          nsnull);
+        nsIScrollableView* scrollableView;
+        if (NS_SUCCEEDED(CallQueryInterface(parentView, &scrollableView))) {
+          nscoord width, height;
+          scrollableView->GetContainerSize(&width, &height);
+          const nsIView* clippedView;
+          scrollableView->GetClipView(&clippedView);
+          nsRect vcr = clippedView->GetBounds();
+          focusRect.width = vcr.width;
+          focusRect.height = vcr.height;
+          nscoord x,y;
+          scrollableView->GetScrollPosition(x, y);
+          focusRect.x += x;
+          focusRect.y += y;
         }
+
+        nsStyleOutline outlineStyle(aPresContext);
+        outlineStyle.SetOutlineStyle(NS_STYLE_BORDER_STYLE_DOTTED);
+        outlineStyle.SetOutlineInvert();
+
+        float p2t = aPresContext->PixelsToTwips();
+        // XXX the CSS border for links is specified as 2px, but it
+        // is only drawn as 1px.  Match this here.
+        nscoord onePixel = NSIntPixelsToTwips(1, p2t);
+
+        nsRect borderInside(focusRect.x + onePixel,
+                            focusRect.y + onePixel,
+                            focusRect.width - 2 * onePixel,
+                            focusRect.height - 2 * onePixel);
+
+        nsCSSRendering::DrawDashedSides(0, aRenderingContext, 
+                                        focusRect, nsnull,
+                                        nsnull, &outlineStyle,
+                                        PR_TRUE, focusRect,
+                                        borderInside, 0, 
+                                        nsnull);
       }
     }
   }
@@ -486,7 +473,7 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
   DO_GLOBAL_REFLOW_COUNT("CanvasFrame", aReflowState.reason);
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
   NS_FRAME_TRACE_REFLOW_IN("CanvasFrame::Reflow");
-  //NS_PRECONDITION(nsnull == aDesiredSize.maxElementSize, "unexpected request");
+  //NS_PRECONDITION(!aDesiredSize.mComputeMEW, "unexpected request");
 
   // Initialize OUT parameter
   aStatus = NS_FRAME_COMPLETE;
@@ -497,12 +484,11 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
   // Check for an incremental reflow
   if (eReflowReason_Incremental == aReflowState.reason) {
     // See if we're the target frame
-    nsIFrame* targetFrame;
-    aReflowState.reflowCommand->GetTarget(targetFrame);
-    if (this == targetFrame) {
+    nsHTMLReflowCommand *command = aReflowState.path->mReflowCommand;
+    if (command) {
       // Get the reflow type
-      nsReflowType  reflowType;
-      aReflowState.reflowCommand->GetType(reflowType);
+      nsReflowType reflowType;
+      command->GetType(reflowType);
 
       switch (reflowType) {
       case eReflowType_ReflowDirty:
@@ -517,12 +503,12 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
       default:
         NS_ASSERTION(PR_FALSE, "unexpected reflow command type");
       }
-
-    } else {
-      nsIFrame* nextFrame;
-      // Get the next frame in the reflow chain
-      aReflowState.reflowCommand->GetNext(nextFrame);
-      NS_ASSERTION(nextFrame == mFrames.FirstChild(), "unexpected next reflow command frame");
+    }
+    else {
+#ifdef DEBUG
+      nsReflowPath::iterator iter = aReflowState.path->FirstChild();
+      NS_ASSERTION(*iter == mFrames.FirstChild(), "unexpected next reflow command frame");
+#endif
     }
   }
 
@@ -538,17 +524,26 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
 
     // We must specify an unconstrained available height, because constrained
     // is only for when we're paginated...
-    nsHTMLReflowState kidReflowState(aPresContext, aReflowState, kidFrame,
-                                     nsSize(aReflowState.availableWidth,
-                                            NS_UNCONSTRAINEDSIZE));
+    nsReflowReason reason;
     if (isDirtyChildReflow) {
       // Note: the only reason the frame would be dirty would be if it had
       // just been inserted or appended
-      kidReflowState.reason = eReflowReason_Initial;
-      kidReflowState.reflowCommand = nsnull;
+      reason = eReflowReason_Initial;
     } else if (isStyleChange) {
-      kidReflowState.reason = eReflowReason_StyleChange;
-      kidReflowState.reflowCommand = nsnull;
+      reason = eReflowReason_StyleChange;
+    } else {
+      reason = aReflowState.reason;
+    }
+
+    nsHTMLReflowState kidReflowState(aPresContext, aReflowState, kidFrame,
+                                     nsSize(aReflowState.availableWidth,
+                                            NS_UNCONSTRAINEDSIZE),
+                                     reason);
+
+    if (eReflowReason_Incremental == aReflowState.reason) {
+      // Restore original kid desired dimensions in case it decides to
+      // reuse them during incremental reflow.
+      kidFrame->SetSize(nsSize(mSavedChildWidth, mSavedChildHeight));
     }
 
     // Reflow the frame
@@ -556,19 +551,20 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
                 kidReflowState.mComputedMargin.left, kidReflowState.mComputedMargin.top,
                 0, aStatus);
 
+    mSavedChildWidth = kidDesiredSize.width;
+    mSavedChildHeight = kidDesiredSize.height;
+
     // The document element's background should cover the entire canvas, so
     // take into account the combined area and any space taken up by
     // absolutely positioned elements
     nsMargin      border;
-    nsFrameState  kidState;
 
     if (!kidReflowState.mStyleBorder->GetBorder(border)) {
       NS_NOTYETIMPLEMENTED("percentage border");
     }
-    kidFrame->GetFrameState(&kidState);
 
     // First check the combined area
-    if (NS_FRAME_OUTSIDE_CHILDREN & kidState) {
+    if (NS_FRAME_OUTSIDE_CHILDREN & kidFrame->GetStateBits()) {
       // The background covers the content area and padding area, so check
       // for children sticking outside the child frame's padding edge
       nscoord paddingEdgeX = kidDesiredSize.width - border.right;
@@ -596,15 +592,14 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
     }
 
     // Complete the reflow and position and size the child frame
-    nsRect  rect(kidReflowState.mComputedMargin.left, kidReflowState.mComputedMargin.top,
-                 kidDesiredSize.width, kidDesiredSize.height);
-    FinishReflowChild(kidFrame, aPresContext, &kidReflowState, kidDesiredSize, rect.x, rect.y, 0);
+    FinishReflowChild(kidFrame, aPresContext, &kidReflowState, kidDesiredSize,
+                      kidReflowState.mComputedMargin.left,
+                      kidReflowState.mComputedMargin.top, 0);
 
     // If the child frame was just inserted, then we're responsible for making sure
     // it repaints
     if (isDirtyChildReflow) {
-      // Damage the area occupied by the deleted frame
-      Invalidate(aPresContext, rect, PR_FALSE);
+      Invalidate(kidFrame->GetOutlineRect() + kidFrame->GetPosition(), PR_FALSE);
     }
 
     // Return our desired size
@@ -619,6 +614,7 @@ CanvasFrame::Reflow(nsIPresContext*          aPresContext,
   }
 
   NS_FRAME_TRACE_REFLOW_OUT("CanvasFrame::Reflow", aStatus);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -642,12 +638,11 @@ CanvasFrame::HandleEvent(nsIPresContext* aPresContext,
       aEvent->message == NS_MOUSE_MIDDLE_BUTTON_UP ||
       aEvent->message == NS_MOUSE_RIGHT_BUTTON_UP ||
       aEvent->message == NS_MOUSE_MOVE ) {
-    nsIFrame *firstChild;
-    nsresult rv = FirstChild(aPresContext,nsnull,&firstChild);
+    nsIFrame *firstChild = GetFirstChild(nsnull);
     //canvas frame needs to pass mouse events to its area frame so that mouse movement
     //and selection code will work properly. this will still have the necessary effects
     //that would have happened if nsFrame::HandleEvent was called.
-    if (NS_SUCCEEDED(rv) && firstChild)
+    if (firstChild)
       firstChild->HandleEvent(aPresContext, aEvent, aEventStatus);
     else
       nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
@@ -666,32 +661,10 @@ CanvasFrame::GetFrameForPoint(nsIPresContext* aPresContext,
   return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND), aFrame);
 }
 
-NS_IMETHODIMP
-CanvasFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+CanvasFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::canvasFrame; 
-  NS_ADDREF(*aType);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-CanvasFrame::AttributeChanged(nsIPresContext* aPresContext,
-                          nsIContent*     aChild,
-                          PRInt32         aNameSpaceID,
-                          nsIAtom*        aAttribute,
-                          PRInt32         aModType, 
-                          PRInt32         aHint)
-{
-// if the background color or image is changing, invalidate the canvas
-  if (aHint > 0){
-    if (aAttribute == nsHTMLAtoms::bgcolor ||
-        aAttribute == nsHTMLAtoms::background ||
-        aAttribute == nsHTMLAtoms::style) {
-      Invalidate(aPresContext,mRect,PR_FALSE);
-    }
-  }
-  return NS_OK;
+  return nsLayoutAtoms::canvasFrame;
 }
 
 NS_IMETHODIMP 
@@ -721,14 +694,4 @@ CanvasFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("Canvas"), aResult);
 }
-
-NS_IMETHODIMP
-CanvasFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  *aResult = sizeof(*this);
-  return NS_OK;
-}
 #endif
