@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Seth Spitzer <sspitzer@netscape.com>
+ *   Dan Mosedale <dmose@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -52,6 +53,11 @@
 #include "nsNetCID.h"
 #include "nsIIOService.h"
 #include "nsIPref.h"
+#include "nsXPCOM.h"
+#include "nsISupportsPrimitives.h"
+#include "nsIPrefBranch.h"
+#include "nsCOMArray.h"
+#include "nsArrayEnumerator.h"
 
 nsAbLDAPDirectory::nsAbLDAPDirectory() :
     nsAbDirectoryRDFResource(),
@@ -117,10 +123,65 @@ nsresult nsAbLDAPDirectory::InitiateConnection ()
     // turn moz-abldapdirectory://ldap_2.servers.nscpphonebook into -> "ldap_2.servers.nscpphonebook.uri"
     nsXPIDLCString URI;
     rv = prefs->CopyCharPref(prefName.get(), getter_Copies(URI));
+    if (NS_FAILED(rv))
+    {
+        /*
+         * A recent change in Mozilla now means that the LDAP Address Book
+         * RDF Resource URI is based on the unique preference name value i.e.  
+         * [moz-abldapdirectory://prefName]
+         * Prior to this valid change it was based on the actual uri i.e. 
+         * [moz-abldapdirectory://host:port/basedn]
+         * Basing the resource on the prefName allows these attributes to 
+         * change. 
+         *
+         * But the uri value was also the means by which third-party
+         * products could integrate with Mozilla's LDAP Address Books
+         * without necessarily having an entry in the preferences file
+         * or more importantly needing to be able to change the
+         * preferences entries. Thus to set the URI Spec now, it is
+         * only necessary to read the uri pref entry, while in the
+         * case where it is not a preference, we need to replace the
+         * "moz-abldapdirectory".
+         */
+        nsCAutoString tempLDAPURL(mURINoQuery);
+        tempLDAPURL.ReplaceSubstring("moz-abldapdirectory:", "ldap:");
+        rv = mURL->SetSpec(tempLDAPURL);
+    }
+    else
+    {
+        rv = mURL->SetSpec(URI);
+    }
     NS_ENSURE_SUCCESS(rv,rv);
-    
-    rv = mURL->SetSpec(URI);
-    NS_ENSURE_SUCCESS(rv, rv);
+
+    // get the login information, if there is any 
+    //
+    rv = prefs->GetCharPref(
+        PromiseFlatCString(
+            Substring(mURINoQuery, kLDAPDirectoryRootLen,
+                      mURINoQuery.Length() - kLDAPDirectoryRootLen)
+            + NS_LITERAL_CSTRING(".auth.dn")).get(),
+        getter_Copies(mLogin));
+    if (NS_FAILED(rv)) {
+        mLogin.Truncate();  // zero out mLogin
+    }
+
+    // get the protocol version, if there is any.  using a string pref
+    // here instead of an int, as protocol versions sometimes have names like
+    // "4bis".
+    //
+    nsXPIDLCString protocolVersion;
+    rv = prefs->GetCharPref(
+        PromiseFlatCString(
+            Substring(mURINoQuery, kLDAPDirectoryRootLen,
+                      mURINoQuery.Length() - kLDAPDirectoryRootLen)
+            + NS_LITERAL_CSTRING(".protocolVersion")).get(),
+        getter_Copies(protocolVersion));
+
+    if (NS_SUCCEEDED(rv) && protocolVersion.Equals("2")) {
+        mProtocolVersion = nsILDAPConnection::VERSION2;
+    }
+    // otherwise we leave mProtocolVersion as the default (see the initializers
+    // for the nsAbLDAPDirectoryQuery class).
 
     mConnection = do_CreateInstance(NS_LDAPCONNECTION_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -142,14 +203,10 @@ NS_IMETHODIMP nsAbLDAPDirectory::GetOperations(PRInt32 *aOperations)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsAbLDAPDirectory::GetChildNodes(nsIEnumerator* *result)
+NS_IMETHODIMP nsAbLDAPDirectory::GetChildNodes(nsISimpleEnumerator* *aResult)
 {
-    nsCOMPtr<nsISupportsArray> array;
-    NS_NewISupportsArray(getter_AddRefs(array));
-    if (!array)
-            return NS_ERROR_OUT_OF_MEMORY;
-
-    return array->Enumerate(result);
+    nsCOMArray<nsIAbDirectory> children;
+    return NS_NewArrayEnumerator(aResult, children);
 }
 
 NS_IMETHODIMP nsAbLDAPDirectory::GetChildCards(nsIEnumerator** result)
@@ -187,7 +244,7 @@ NS_IMETHODIMP nsAbLDAPDirectory::GetChildCards(nsIEnumerator** result)
       localDirectoryURI = NS_LITERAL_CSTRING("moz-abmdbdirectory://") + fileName + NS_LITERAL_CSTRING("?") + mQueryString;
       
       nsCOMPtr <nsIRDFResource> resource;
-      rv = rdfService->GetResource(localDirectoryURI.get(), getter_AddRefs(resource));
+      rv = rdfService->GetResource(localDirectoryURI, getter_AddRefs(resource));
       NS_ENSURE_SUCCESS(rv, rv);
       
       nsCOMPtr <nsIAbDirectory> directory = do_QueryInterface(resource, &rv);
@@ -416,6 +473,27 @@ NS_IMETHODIMP nsAbLDAPDirectory::GetIsRemote(PRBool *aIsRemote)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsAbLDAPDirectory::GetIsSecure(PRBool *aIsSecure)
+{
+  NS_ENSURE_ARG_POINTER(aIsSecure);
+
+  nsresult rv;
+  nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // use mURINoQuery to get a prefName
+  nsCAutoString prefName;
+  prefName = nsDependentCString(mURINoQuery.get() + kLDAPDirectoryRootLen) + NS_LITERAL_CSTRING(".uri");
+  
+  nsXPIDLCString URI;
+  rv = prefBranch->GetCharPref(prefName.get(), getter_Copies(URI));
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  // to determine if this is a secure directory, check if the uri is ldaps:// or not
+  *aIsSecure = (strncmp(URI.get(), "ldaps:", 6) == 0);
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAbLDAPDirectory::GetSearchDuringLocalAutocomplete(PRBool *aSearchDuringLocalAutocomplete)
 {
   NS_ENSURE_ARG_POINTER(aSearchDuringLocalAutocomplete);
