@@ -166,7 +166,6 @@ const ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[EXPORT_ATTRIBUTES_TABL
 //
 nsAddressBook::nsAddressBook()
 {
-  mDocShell = nsnull;
 }
 
 nsAddressBook::~nsAddressBook()
@@ -260,25 +259,6 @@ nsresult nsAddressBook::DoCommand(nsIRDFDataSource* db,
   return rv;
 }
 
-NS_IMETHODIMP nsAddressBook::SetDocShellWindow(nsIDOMWindowInternal *aWin)
-{
-  NS_PRECONDITION(aWin != nsnull, "null ptr");
-  if (!aWin)
-    return NS_ERROR_NULL_POINTER;
- 
-  nsCOMPtr<nsIScriptGlobalObject> globalObj( do_QueryInterface(aWin) );
-  if (!globalObj) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // mDocShell is a weak reference
-  mDocShell = globalObj->GetDocShell();
-  if (!mDocShell)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsAddressBook::GetAbDatabaseFromURI(const char *aURI, nsIAddrDatabase **aDB)
 {
   NS_ENSURE_ARG_POINTER(aURI);
@@ -1348,8 +1328,9 @@ enum ADDRESSBOOK_EXPORT_FILE_TYPE
  TAB_EXPORT_TYPE = 2
 };
 
-NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
+NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIDOMWindowInternal *aParentWin, nsIAbDirectory *aDirectory)
 {
+  NS_ENSURE_ARG_POINTER(aParentWin);
   nsresult rv;
 
   nsCOMPtr<nsIFilePicker> filePicker = do_CreateInstance("@mozilla.org/filepicker;1", &rv);
@@ -1360,31 +1341,42 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
   nsCOMPtr<nsIStringBundle> bundle;
   rv = bundleService->CreateBundle("chrome://messenger/locale/addressbook/addressBook.properties", getter_AddRefs(bundle));
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
+  nsCOMPtr<nsIScriptGlobalObject> globalObj = do_QueryInterface(aParentWin, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIDocShell> docShell = globalObj->GetDocShell();
+  if (!docShell)
+    return NS_ERROR_NULL_POINTER;
+
+  nsCOMPtr<nsIDOMWindow> parentWindow = do_GetInterface(docShell);
+  if (!parentWindow)
+    return NS_NOINTERFACE;
+
   nsXPIDLString title;
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("ExportAddressBookTitle").get(), getter_Copies(title));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->Init(nsnull, title.get(), nsIFilePicker::modeSave);
+  rv = filePicker->Init(parentWindow, title, nsIFilePicker::modeSave);
   NS_ENSURE_SUCCESS(rv, rv);
  
   nsXPIDLString filterString;
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("LDIFFiles").get(), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.ldi; *.ldif").get());
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.ldi; *.ldif"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("CSVFiles").get(), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.csv").get());
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.csv"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("TABFiles").get(), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.tab; *.txt").get());
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   PRInt16 dialogResult;
