@@ -18,7 +18,7 @@
  * Rights Reserved.
  *
  * Contributor(s):
- *   Jan Varga (varga@utcru.sk)
+ *   Jan Varga (varga@nixcorp.com)
  *   Hkan Waara (hwaara@chello.se)
  */
 
@@ -179,7 +179,15 @@ function MsgCompactFolder(isAll)
 function MsgFolderProperties() 
 {
 	var preselectedURI = GetSelectedFolderURI();
-	var serverType = GetMsgFolderFromUri(preselectedURI, true).server.type;
+  var msgFolder = GetMsgFolderFromUri(preselectedURI, true);
+
+  // if a server is selected, view settings for that account
+  if (msgFolder.isServer) {
+    MsgAccountManager(null);
+    return;
+  }
+
+	var serverType = msgFolder.server.type;
 	var folderTree = GetFolderTree();
 
 	var name = GetFolderNameFromUri(preselectedURI, folderTree);
@@ -201,16 +209,11 @@ function FolderProperties(name, uri)
 
 function MsgToggleMessagePane()
 {
-	//OnClickThreadAndMessagePaneSplitter is based on the value before the splitter is toggled.
-	OnClickThreadAndMessagePaneSplitterGrippy();
+  //OnClickThreadAndMessagePaneSplitter is based on the value before the splitter is toggled.
+  OnClickThreadAndMessagePaneSplitterGrippy();
   MsgToggleSplitter("threadpane-splitter");
 }
 
-function MsgToggleFolderPane()
-{
-    MsgToggleSplitter("sidebarsplitter");
-}
-
 function MsgToggleSplitter(id)
 {
     var splitter = document.getElementById(id);
@@ -221,42 +224,6 @@ function MsgToggleSplitter(id)
         splitter.setAttribute("state", "collapsed")
 }
 
-
-function NotifyQuitApplication()
-{
-	var ObserverService = Components.classes["@mozilla.org/observer-service;1"].getService();
-	ObserverService = ObserverService.QueryInterface(Components.interfaces.nsIObserverService);
-	if (ObserverService)
-	{
-		try 
-		{
-			ObserverService.notifyObservers(null, "quit-application", null);
-		} 
-		catch (ex) 
-		{
-			// dump("no observer found \n");
-		}
-	}
-}
-
-function LastToClose()
-{
-	var windowManager = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService();
-	var	windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
-	var enumerator = windowManagerInterface.getEnumerator( null );
-    var count = 0;
-
-	while ( enumerator.hasMoreElements() && count < 2 )
-	{
-		var  windowToClose = enumerator.getNext();
-        count++;
-    }
-    if (count == 1)
-        return true;
-    else
-        return false;
-}
-
 function MsgSetFolderCharset() 
 {
   MsgFolderProperties() 
