@@ -166,7 +166,7 @@ struct PRJob {
 				PR_END_MACRO
 
 static void delete_job(PRJob *jobp);
-static PRThreadPool * alloc_threadpool();
+static PRThreadPool * alloc_threadpool(void);
 static PRJob * alloc_job(PRBool joinable, PRThreadPool *tp);
 static void notify_ioq(PRThreadPool *tp);
 static void notify_timerq(PRThreadPool *tp);
@@ -599,11 +599,11 @@ delete_threadpool(PRThreadPool *tp)
 }
 
 static PRThreadPool *
-alloc_threadpool()
+alloc_threadpool(void)
 {
 PRThreadPool *tp;
 
-	tp = PR_CALLOC(sizeof(*tp));
+	tp = (PRThreadPool *) PR_CALLOC(sizeof(*tp));
 	if (NULL == tp)
 		goto failed;
 	tp->jobq.lock = PR_NewLock();
