@@ -36,6 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsURILoader.h"
+#include "nsAutoPtr.h"
 #include "nsIURIContentListener.h"
 #include "nsIContentHandler.h"
 #include "nsILoadGroup.h"
@@ -71,8 +72,19 @@
 #include "nsICategoryManager.h"
 #include "nsCExternalHandlerService.h" // contains contractids for the helper app service
 
-static NS_DEFINE_CID(kStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
+#include "nsIMIMEHeaderParam.h"
+#include "nsNetCID.h"
 
+#include "nsMimeTypes.h"
+
+static NS_DEFINE_CID(kStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
+#ifdef PR_LOGGING
+PRLogModuleInfo* nsURILoader::mLog = nsnull;
+#endif
+
+#define LOG(args) PR_LOG(nsURILoader::mLog, PR_LOG_DEBUG, args)
+#define LOG_ERROR(args) PR_LOG(nsURILoader::mLog, PR_LOG_ERROR, args)
+#define LOG_ENABLED() PR_LOG_TEST(nsURILoader::mLog, PR_LOG_DEBUG)
 
 /**
  * The nsDocumentOpenInfo contains the state required when a single
@@ -88,7 +100,8 @@ public:
 
   // Real constructor
   nsDocumentOpenInfo(nsISupports* aWindowContext,
-                     PRBool aIsContentPreferred);
+                     PRBool aIsContentPreferred,
+                     nsURILoader* aURILoader);
 
   NS_DECL_ISUPPORTS
 
@@ -102,9 +115,19 @@ public:
   // aOutContentType into the StreamListener chain.  DO NOT call it if the two
   // types are the same, since no conversion is needed in that case.
   nsresult ConvertData(nsIRequest *request,
+                       nsIURIContentListener *aListener,
                        const nsACString & aSrcContentType,
                        const nsACString & aOutContentType);
 
+  /**
+   * Function to attempt to use aListener to handle the load.  If
+   * PR_TRUE is returned, nothing else needs to be done; if PR_FALSE
+   * is returned, then a different way of handling the load should be
+   * tried.
+   */
+  PRBool TryContentListener(nsIURIContentListener* aListener,
+                            nsIChannel* aChannel);
+
   // nsIRequestObserver methods:
   NS_DECL_NSIREQUESTOBSERVER
 
@@ -115,14 +138,45 @@ protected:
   virtual ~nsDocumentOpenInfo();
 
 protected:
+  /**
+   * The first content listener to try dispatching data to.  Typically
+   * the listener associated with the entity that originated the load.
+   */
   nsCOMPtr<nsIURIContentListener> m_contentListener;
+
+  /**
+   * The stream listener to forward nsIStreamListener notifications
+   * to.  This is set once the load is dispatched.
+   */
   nsCOMPtr<nsIStreamListener> m_targetStreamListener;
+
+  /**
+   * A pointer to the entity that originated the load.  This should
+   * implement nsIInterfaceRequestor; we depend on getting things like
+   * nsIURIContentListeners, nsIDOMWindows, etc off of it.
+   */
   nsCOMPtr<nsISupports> m_originalContext;
+
+  /**
+   * Boolean to pass to CanHandleContent (also determines whether we
+   * use CanHandleContent or IsPreferred).
+   */
   PRBool mIsContentPreferred;
+
+  /**
+   * The type of the data we will be trying to dispatch.
+   */
+  nsCString mContentType;
+
+  /**
+   * Reference to the URILoader service so we can access its list of
+   * nsIURIContentListeners.
+   */
+  nsRefPtr<nsURILoader> mURILoader;
 };
 
-NS_IMPL_THREADSAFE_ADDREF(nsDocumentOpenInfo);
-NS_IMPL_THREADSAFE_RELEASE(nsDocumentOpenInfo);
+NS_IMPL_THREADSAFE_ADDREF(nsDocumentOpenInfo)
+NS_IMPL_THREADSAFE_RELEASE(nsDocumentOpenInfo)
 
 NS_INTERFACE_MAP_BEGIN(nsDocumentOpenInfo)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIRequestObserver)
@@ -136,9 +190,11 @@ nsDocumentOpenInfo::nsDocumentOpenInfo()
 }
 
 nsDocumentOpenInfo::nsDocumentOpenInfo(nsISupports* aWindowContext,
-                                       PRBool aIsContentPreferred)
+                                       PRBool aIsContentPreferred,
+                                       nsURILoader* aURILoader)
   : m_originalContext(aWindowContext),
-    mIsContentPreferred(aIsContentPreferred)
+    mIsContentPreferred(aIsContentPreferred),
+    mURILoader(aURILoader)
 {
 }
 
@@ -148,6 +204,8 @@ nsDocumentOpenInfo::~nsDocumentOpenInfo()
 
 nsresult nsDocumentOpenInfo::Open(nsIChannel *aChannel)
 {
+  LOG(("[0x%p] nsDocumentOpenInfo::Open", this));
+    
   // this method is not complete!!! Eventually, we should first go
   // to the content listener and ask them for a protocol handler...
   // if they don't give us one, we need to go to the registry and get
@@ -162,14 +220,13 @@ nsresult nsDocumentOpenInfo::Open(nsIChannel *aChannel)
   if (NS_FAILED(rv)) return rv;
 
   // now just open the channel!
-  if (aChannel){
-    rv = aChannel->AsyncOpen(this, nsnull);
-  }
+  rv = aChannel->AsyncOpen(this, nsnull);
 
   // no content from this load - that's OK.
   if (rv == NS_ERROR_DOM_RETVAL_UNDEFINED ||
       rv == NS_ERROR_NO_CONTENT) {
-      rv = NS_OK;
+    LOG(("  rv is NS_ERROR_DOM_RETVAL_UNDEFINED or NS_ERROR_NO_CONTENT -- doing nothing"));
+    rv = NS_OK;
   }
 
   return rv;
@@ -177,15 +234,18 @@ nsresult nsDocumentOpenInfo::Open(nsIChannel *aChannel)
 
 NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupports * aCtxt)
 {
+  LOG(("[0x%p] nsDocumentOpenInfo::OnStartRequest", this));
+  
   nsresult rv = NS_OK;
 
   //
   // Deal with "special" HTTP responses:
-  //
-  // - In the case of a 204 (No Content)  or 205 (Reset Content) response, do not try to find a
-  //   content handler.  Just return.  This causes the request to be
-  //   ignored.
-  //
+  // 
+  // - In the case of a 204 (No Content) or 205 (Reset Content) response, do
+  //   not try to find a content handler.  Return NS_BINDING_ABORTED to cancel
+  //   the request.  This has the effect of ensuring that the DocLoader does
+  //   not try to interpret this as a real request.
+  // 
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request, &rv));
 
   if (NS_SUCCEEDED(rv)) {
@@ -194,12 +254,16 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupport
     rv = httpChannel->GetResponseStatus(&responseCode);
 
     if (NS_FAILED(rv)) {
+      LOG_ERROR(("  Failed to get HTTP response status"));
+      
       // behave as in the canceled case
       return NS_OK;
     }
 
+    LOG(("  HTTP response status: %d", responseCode));
+
     if (204 == responseCode || 205 == responseCode) {
-      return NS_OK;
+      return NS_BINDING_ABORTED;
     }
   }
 
@@ -214,6 +278,8 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupport
   if (NS_FAILED(rv)) return rv;
 
   if (NS_FAILED(status)) {
+    LOG_ERROR(("  Request failed, status: 0x%08X", rv));
+  
     //
     // The transaction has already reported an error - so it will be torn
     // down. Therefore, it is not necessary to return an error code...
@@ -221,9 +287,48 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupport
     return NS_OK;
   }
 
+  if (httpChannel && mContentType.IsEmpty()) {
+    // This is our initial dispatch, and this is an HTTP channel.  Check for
+    // the text/plain mess.
+    nsCAutoString contentType;
+    httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Content-Type"),
+                                   contentType);
+    // Make sure to do a case-sensitive exact match comparison here.  Apache
+    // 1.x just sends text/plain for "unknown", while Apache 2.x sends
+    // text/plain with a ISO-8859-1 charset.  Debian's Apache version, just to
+    // be different, sends text/plain with iso-8859-1 charset.  Don't do
+    // general case-insensitive comparison, since we really want to apply this
+    // crap as rarely as we can.
+    if (contentType.Equals(NS_LITERAL_CSTRING("text/plain")) ||
+        contentType.Equals(
+             NS_LITERAL_CSTRING("text/plain; charset=ISO-8859-1")) ||
+        contentType.Equals(
+             NS_LITERAL_CSTRING("text/plain; charset=iso-8859-1"))) {
+      // Check whether we have content-encoding.  If we do, don't try to detect
+      // the type, since that will lead to the content being automatically
+      // decompressed....
+      nsCAutoString contentEncoding;
+      httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Content-Encoding"),
+                                     contentEncoding);
+      if (contentEncoding.IsEmpty()) {
+        // OK, this is initial dispatch of an HTTP response and its Content-Type
+        // header is exactly "text/plain".  We need to check whether this is
+        // really text....
+        LOG(("  Possibly bogus text/plain; resetting type to " APPLICATION_MAYBE_TEXT));
+        httpChannel->SetContentType(NS_LITERAL_CSTRING(APPLICATION_MAYBE_TEXT));
+      }
+    }
+  }
+  
   rv = DispatchContent(request, aCtxt);
+
+  LOG(("  After dispatch, m_targetStreamListener: 0x%p", m_targetStreamListener.get()));
+  
   if (m_targetStreamListener)
     rv = m_targetStreamListener->OnStartRequest(request, aCtxt);
+
+  LOG(("  OnStartRequest returning: 0x%08X", rv));
+  
   return rv;
 }
 
@@ -243,11 +348,16 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnDataAvailable(nsIRequest *request, nsISuppor
 NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest *request, nsISupports *aCtxt, 
                                                 nsresult aStatus)
 {
+  LOG(("[0x%p] nsDocumentOpenInfo::OnStopRequest", this));
+  
   if ( m_targetStreamListener)
   {
     nsCOMPtr<nsIStreamListener> listener(m_targetStreamListener);
 
+    // If this is a multipart stream, we could get another
+    // OnStartRequest after this... reset state.
     m_targetStreamListener = 0;
+    mContentType.Truncate();
     listener->OnStopRequest(request, aCtxt, aStatus);
   }
 
@@ -260,29 +370,29 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest *request, nsISupports
 
 nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports * aCtxt)
 {
+  LOG(("[0x%p] nsDocumentOpenInfo::DispatchContent for type '%s'", this, mContentType.get()));
+
+  NS_PRECONDITION(!m_targetStreamListener,
+                  "Why do we already have a target stream listener?");
+  
   nsresult rv;
   nsCOMPtr<nsIChannel> aChannel = do_QueryInterface(request);
   if (!aChannel) {
-      return NS_ERROR_FAILURE;
+    LOG_ERROR(("  Request is not a channel.  Bailing."));
+    return NS_ERROR_FAILURE;
   }
 
-  nsCAutoString contentType;
-  rv = aChannel->GetContentType(contentType);
-  if (NS_FAILED(rv)) return rv;
-
-  // go to the uri dispatcher and give them our stuff...
-  nsCOMPtr<nsIURILoader> uriLoader;
-  uriLoader = do_GetService(NS_URI_LOADER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) {
-    return rv;
+  NS_NAMED_LITERAL_CSTRING(anyType, "*/*");
+  if (mContentType.IsEmpty() || mContentType == anyType) {
+    rv = aChannel->GetContentType(mContentType);
+    if (NS_FAILED(rv)) return rv;
+    LOG(("  Got type from channel: '%s'", mContentType.get()));
   }
 
   // Check whether the data should be forced to be handled externally.  This
   // could happen because the Content-Disposition header is set so, or, in the
   // future, because the user has specified external handling for the MIME
   // type.
-  // XXXbz we need a utility function in necko for parsing content-disposition
-  // headers, methinks.
   PRBool forceExternalHandling = PR_FALSE;
   nsCAutoString disposition;
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
@@ -300,43 +410,33 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports *
     }
   }
 
+  LOG(("  Disposition header: '%s'", disposition.get()));
+
   if (NS_SUCCEEDED(rv) && !disposition.IsEmpty())
   {
-    nsCAutoString::const_iterator start, end;
-    disposition.BeginReading(start);
-    disposition.EndReading(end);
-    // skip leading whitespace
-    while (start != end && nsCRT::IsAsciiSpace(*start))
+    nsCOMPtr<nsIMIMEHeaderParam> mimehdrpar = do_GetService(NS_MIMEHEADERPARAM_CONTRACTID, &rv);
+    if (NS_SUCCEEDED(rv))
     {
-      ++start;
-    }
-    nsCAutoString::const_iterator iter = start;
-    // walk forward till we hit the next whitespace, semicolon, or
-    // equals sign
-    while (iter != end && *iter != ';' && *iter != '=' &&
-           !nsCRT::IsAsciiSpace(*iter))
-    {
-      ++iter;
-    }
-      
-    if (start != iter)
-    {
-      const nsACString & dispToken = Substring(start, iter);
+      nsAutoString dispToken;
+      // Get the disposition type
+      rv = mimehdrpar->GetParameter(disposition, "", EmptyCString(), 
+                                    PR_FALSE, nsnull, dispToken);
       // RFC 2183, section 2.8 says that an unknown disposition
       // value should be treated as "attachment"
-      if (!dispToken.Equals(NS_LITERAL_CSTRING("inline"),
-                            nsCaseInsensitiveCStringComparator()) &&
+      if (NS_FAILED(rv) || 
+          (!dispToken.EqualsIgnoreCase("inline") &&
           // Broken sites just send
           // Content-Disposition: filename="file"
           // without a disposition token... screen those out.
-          !dispToken.Equals(NS_LITERAL_CSTRING("filename"),
-                            nsCaseInsensitiveCStringComparator()))
-      {
+           !dispToken.EqualsIgnoreCase("filename", 8)) &&
+          // Also in use is Content-Disposition: name="file"
+           !dispToken.EqualsIgnoreCase("name", 4))
         // We have a content-disposition of "attachment" or unknown
         forceExternalHandling = PR_TRUE;
-      }
     }
   }
+
+  LOG(("  forceExternalHandling: %s", forceExternalHandling ? "yes" : "no"));
     
   // We're going to try to find a contentListener that can handle our data
   nsCOMPtr<nsIURIContentListener> contentListener;
@@ -346,30 +446,100 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports *
   if (!forceExternalHandling)
   {
     //
-    // First step:  See if any nsIURIContentListener prefers to handle this
-    //              content type, passing m_contentListener as the first
-    //              listener to try.
+    // First step: See whether m_contentListener wants to handle this
+    // content type.
     //
-    PRBool abortDispatch = PR_FALSE;
-    rv = uriLoader->DispatchContent(contentType.get(),
-                                    mIsContentPreferred, 
-                                    request, aCtxt, 
-                                    m_contentListener, 
-                                    m_originalContext,
-                                    getter_Copies(desiredContentType), 
-                                    getter_AddRefs(contentListener),
-                                    &abortDispatch);  
+    if (m_contentListener && TryContentListener(m_contentListener, aChannel)) {
+      LOG(("  Success!  Our default listener likes this type"));
+      // All done here
+      return NS_OK;
+    }
 
-    // if the uri loader says to abort the dispatch then someone
-    // else must have stepped in and taken over for us...so stop..
+    //
+    // Second step: See whether some other registered listener wants
+    // to handle this content type.
+    //
+    PRInt32 count = mURILoader->m_listeners.Count();
+    nsCOMPtr<nsIURIContentListener> listener;
+    for (PRInt32 i = 0; i < count; i++) {
+      listener = do_QueryReferent(mURILoader->m_listeners[i]);
+      if (listener) {
+        if (TryContentListener(listener, aChannel)) {
+          LOG(("  Found listener registered on the URILoader"));
+          return NS_OK;
+        }
+      } else {
+        // remove from the listener list, reset i and update count
+        mURILoader->m_listeners.RemoveObjectAt(i--);
+        --count;
+      }
+    }
 
-    if (abortDispatch) return NS_OK;
+    //
+    // Third step: Try to find a content listener that has not yet had
+    // the chance to register, as it is contained in a not-yet-loaded
+    // module, but which has registered a contract ID.
+    //
+    nsCOMPtr<nsICategoryManager> catman =
+      do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
+    if (catman) {
+      nsXPIDLCString contractidString;
+      rv = catman->GetCategoryEntry(NS_CONTENT_LISTENER_CATEGORYMANAGER_ENTRY,
+                                    mContentType.get(),
+                                    getter_Copies(contractidString));
+      if (NS_SUCCEEDED(rv) && !contractidString.IsEmpty()) {
+        LOG(("  Listener contractid for '%s' is '%s'",
+             mContentType.get(), contractidString.get()));
+
+        listener = do_CreateInstance(contractidString);
+        LOG(("  Listener from category manager: 0x%p", listener.get()));
+        
+        if (listener && TryContentListener(listener, aChannel)) {
+          LOG(("  Listener from category manager likes this type"));
+          return NS_OK;
+        }
+      }
+    }
+
+    //
+    // Fourth step: try to find an nsIContentHandler for our type.
+    //
+    nsCAutoString handlerContractID (NS_CONTENT_HANDLER_CONTRACTID_PREFIX);
+    handlerContractID += mContentType;
+
+    nsCOMPtr<nsIContentHandler> contentHandler =
+      do_CreateInstance(handlerContractID.get());
+    if (contentHandler) {
+      LOG(("  Content handler found"));
+      rv = contentHandler->HandleContent(mContentType.get(), "view",
+                                         m_originalContext, request);
+      // XXXbz returning an error code to represent handling the
+      // content is just bizarre!
+      if (rv != NS_ERROR_WONT_HANDLE_CONTENT) {
+        if (NS_FAILED(rv)) {
+          // The content handler has unexpectedly failed.  Cancel the request
+          // just in case the handler didn't...
+          LOG(("  Content handler failed.  Aborting load"));
+          request->Cancel(rv);
+        }
+#ifdef PR_LOGGING
+        else {
+          LOG(("  Content handler taking over load"));
+        }
+#endif
+
+        return rv;
+      }
+    }
   }
 
+  NS_ASSERTION(!m_targetStreamListener,
+               "If we found a listener, why are we not using it?");
+  
   //
-  // Second step:  If no listener prefers this type, see if any stream
-  //               decoders exist to transform this content type into
-  //               some other.
+  // Fifth step:  If no listener prefers this type, see if any stream
+  //              converters exist to transform this content type into
+  //              some other.
   //
 
   // We always want to do this, since even content being forced to
@@ -379,129 +549,70 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports *
   // it in our Accept header and got confused.
   // XXXbz have to be careful here; may end up in some sort of bizarre infinite
   // decoding loop.
-  NS_NAMED_LITERAL_CSTRING(anyType, "*/*");
-  if (!contentListener && contentType != anyType)
-  {
-    ConvertData(request, contentType, anyType);
-    
-    if (m_targetStreamListener) {
+  if (mContentType != anyType) {
+    rv = ConvertData(request, m_contentListener, mContentType, anyType);
+    if (NS_FAILED(rv)) {
+      m_targetStreamListener = nsnull;
+    } else if (m_targetStreamListener) {
       // We found a converter for this MIME type.  We'll just pump data into it
       // and let the downstream nsDocumentOpenInfo handle things.
+      LOG(("  Converter taking over now"));
       return NS_OK;
     }
   }
-
-  // Step 3:
+      
+  // Sixth step:
   //
-  // Now we try to get an actual downstream nsIStreamListener to give data to.
-  nsCOMPtr<nsIStreamListener> contentStreamListener;
-  if (contentListener)
-  {
-    // We need to first figure out if we are retargeting the load
-    // to a content listener that is different from the one that
-    // originated the request....if so, set
-    // LOAD_RETARGETED_DOCUMENT_URI on the channel.
-    
-    if (contentListener != m_contentListener)
-    {
-      // we must be retargeting...so set an appropriate flag on
-      // the channel
-      nsLoadFlags loadFlags = 0;
-      aChannel->GetLoadFlags(&loadFlags);
-      loadFlags |= nsIChannel::LOAD_RETARGETED_DOCUMENT_URI;
-      aChannel->SetLoadFlags(loadFlags);
-    }
+  // All attempts to dispatch this content have failed.  Just pass it off to
+  // the helper app service.
+  //
+  nsCOMPtr<nsIExternalHelperAppService> helperAppService =
+    do_GetService(NS_EXTERNALHELPERAPPSERVICE_CONTRACTID, &rv);
+  if (helperAppService) {
+    LOG(("  Passing load off to helper app service"));
 
-    const char* contentTypeToUse;
-    if (desiredContentType)
-      contentTypeToUse = desiredContentType.get();
-    else
-      contentTypeToUse = contentType.get();
+    // Set these flags to indicate that the channel has been targeted and that
+    // we are not using the original consumer.
+    nsLoadFlags loadFlags = 0;
+    request->GetLoadFlags(&loadFlags);
+    request->SetLoadFlags(loadFlags | nsIChannel::LOAD_RETARGETED_DOCUMENT_URI
+                                    | nsIChannel::LOAD_TARGETED);
 
-    PRBool bAbortProcess = PR_FALSE;     
-    rv = contentListener->DoContent(contentTypeToUse,
-                                    mIsContentPreferred,
-                                    request,
-                                    getter_AddRefs(contentStreamListener),
-                                    &bAbortProcess);
-
-    // Do not continue loading if nsIURIContentListener::DoContent(...)
-    // fails - It means that an unexpected error occurred...
-    //
-    // If bAbortProcess is TRUE then the listener is doing all the work from
-    // here...we are done!!!
-    if (NS_FAILED(rv) || bAbortProcess) {
-      return rv;
-    }
-  }
-
-  if (!contentStreamListener)
-  {
-    // Step 4:
-    //
-    // All attempts to dispatch this content have failed.  Just pass it off to
-    // the helper app service.
-
-    nsCOMPtr<nsIExternalHelperAppService> helperAppService =
-      do_GetService(NS_EXTERNALHELPERAPPSERVICE_CONTRACTID, &rv);
-    if (helperAppService)
-    {
-      rv = helperAppService->DoContent(contentType.get(),
-                                       request,
-                                       m_originalContext,
-                                       getter_AddRefs(contentStreamListener));
-      if (NS_SUCCEEDED(rv) && contentStreamListener) {
-        m_targetStreamListener = contentStreamListener;
-        return NS_OK;
-      }
+    rv = helperAppService->DoContent(mContentType.get(),
+                                     request,
+                                     m_originalContext,
+                                     getter_AddRefs(m_targetStreamListener));
+    if (NS_FAILED(rv)) {
+      request->SetLoadFlags(loadFlags);
+      m_targetStreamListener = nsnull;
     }
   }
       
-  if (!contentStreamListener) {
-    // Something failed somewhere.... Otherwise the helper app service would
-    // have taken over by now.
-    NS_ASSERTION(NS_FAILED(rv),
-                 "There is no way we should be successful at this point");
-    return rv;
-  }
-
-  if (desiredContentType.IsEmpty() ||
-      contentType == desiredContentType ||
-      contentType == anyType) {
-    // All set.  Just use this contentStreamListener
-    m_targetStreamListener = contentStreamListener;
-    return NS_OK;
-  }
-
-  // Now we have to convert.  Set m_contentListener to contentListener so that
-  // the downstream nsDocumentOpenInfo will try that listener first (since
-  // this contentListener expressed interest in handling the decoded version
-  // of this data).
-  m_contentListener = contentListener;
-  
-  // OK, now bounce the data over to contentStreamListener.
-  rv = ConvertData(request, contentType, desiredContentType);
-
-  // Reinitialize our content listener in case this is a multipart stream --
-  // we wouldn't want to hand over a different part of the stream to the same
-  // content listener as wanted this part.
-  m_contentListener = do_GetInterface(m_originalContext);
-  
+  NS_ASSERTION(m_targetStreamListener || NS_FAILED(rv),
+               "There is no way we should be successful at this point without a m_targetStreamListener");
   return rv;
 }
 
 nsresult
 nsDocumentOpenInfo::ConvertData(nsIRequest *request,
+                                nsIURIContentListener* aListener,
                                 const nsACString& aSrcContentType,
                                 const nsACString& aOutContentType)
 {
+  LOG(("[0x%p] nsDocumentOpenInfo::ConvertData from '%s' to '%s'", this,
+       PromiseFlatCString(aSrcContentType).get(),
+       PromiseFlatCString(aOutContentType).get()));
+
   NS_PRECONDITION(aSrcContentType != aOutContentType,
-                  "RetargetOutput called when the two types are the same!");
+                  "ConvertData called when the two types are the same!");
   nsresult rv = NS_OK;
 
   nsCOMPtr<nsIStreamConverterService> StreamConvService = 
     do_GetService(kStreamConverterServiceCID, &rv);
   if (NS_FAILED(rv)) return rv;
+
+  LOG(("  Got converter service"));
+  
   NS_ConvertASCIItoUCS2 from_w(aSrcContentType);
   NS_ConvertASCIItoUCS2 to_w(aOutContentType);
       
@@ -514,15 +625,24 @@ nsDocumentOpenInfo::ConvertData(nsIRequest *request,
   // intermediate instance is used to target these "decoded" streams...
   //
   nsCOMPtr<nsDocumentOpenInfo> nextLink =
-    new nsDocumentOpenInfo(m_originalContext, mIsContentPreferred);
+    new nsDocumentOpenInfo(m_originalContext, mIsContentPreferred, mURILoader);
   if (!nextLink) return NS_ERROR_OUT_OF_MEMORY;
 
+  LOG(("  Downstream DocumentOpenInfo would be: 0x%p", nextLink.get()));
+  
   // Make sure nextLink starts with the contentListener that said it wanted the
   // results of this decode.
-  nextLink->m_contentListener = m_contentListener;
+  nextLink->m_contentListener = aListener;
   // Also make sure it has to look for a stream listener to pump data into.
   nextLink->m_targetStreamListener = nsnull;
 
+  // Make sure that nextLink treats the data as aOutContentType when
+  // dispatching; that way even if the stream converters don't change the type
+  // on the channel we will still do the right thing.  If aOutContentType is
+  // */*, that's OK -- that will just indicate to nextLink that it should get
+  // the type off the channel.
+  nextLink->mContentType = aOutContentType;
+
   // The following call sets m_targetStreamListener to the input end of the
   // stream converter and sets the output end of the stream converter to
   // nextLink.  As we pump data into m_targetStreamListener the stream
@@ -534,20 +654,118 @@ nsDocumentOpenInfo::ConvertData(nsIRequest *request,
                                              getter_AddRefs(m_targetStreamListener));
 }
 
+PRBool
+nsDocumentOpenInfo::TryContentListener(nsIURIContentListener* aListener,
+                                       nsIChannel* aChannel)
+{
+  LOG(("[0x%p] nsDocumentOpenInfo::TryContentListener; mIsContentPreferred = %s",
+       this, mIsContentPreferred ? "true" : "false"));
+
+  NS_PRECONDITION(aListener, "Must have a non-null listener");
+  NS_PRECONDITION(aChannel, "Must have a channel");
+  
+  PRBool listenerWantsContent = PR_FALSE;
+  nsXPIDLCString typeToUse;
+  
+  if (mIsContentPreferred) {
+    aListener->IsPreferred(mContentType.get(),
+                           getter_Copies(typeToUse),
+                           &listenerWantsContent);
+  } else {
+    aListener->CanHandleContent(mContentType.get(), PR_FALSE,
+                                getter_Copies(typeToUse),
+                                &listenerWantsContent);
+  }
+  if (!listenerWantsContent) {
+    LOG(("  Listener is not interested"));
+    return PR_FALSE;
+  }
+
+  if (!typeToUse.IsEmpty() && typeToUse != mContentType) {
+    // Need to do a conversion here.
+
+    nsresult rv = ConvertData(aChannel, aListener, mContentType, typeToUse);
+
+    if (NS_FAILED(rv)) {
+      // No conversion path -- we don't want this listener, if we got one
+      m_targetStreamListener = nsnull;
+    }
+
+    LOG(("  Found conversion: %s", m_targetStreamListener ? "yes" : "no"));
+    
+    // m_targetStreamListener is now the input end of the converter, and we can
+    // just pump the data in there, if it exists.  If it does not, we need to
+    // try other nsIURIContentListeners.
+    return m_targetStreamListener != nsnull;
+  }
+
+  // At this point, aListener wants data of type mContentType.  Let 'em have
+  // it.  But first, if we are retargeting, set an appropriate flag on the
+  // channel
+  nsLoadFlags loadFlags = 0;
+  aChannel->GetLoadFlags(&loadFlags);
+
+  // Set this flag to indicate that the channel has been targeted at a final
+  // consumer.  This load flag is tested in nsDocLoader::OnProgress.
+  nsLoadFlags newLoadFlags = nsIChannel::LOAD_TARGETED;
+
+  nsCOMPtr<nsIURIContentListener> originalListener =
+    do_GetInterface(m_originalContext);
+  if (originalListener != aListener) {
+    newLoadFlags |= nsIChannel::LOAD_RETARGETED_DOCUMENT_URI;
+  }
+  aChannel->SetLoadFlags(loadFlags | newLoadFlags);
+  
+  PRBool abort = PR_FALSE;
+  nsresult rv = aListener->DoContent(mContentType.get(),
+                                     mIsContentPreferred,
+                                     aChannel,
+                                     getter_AddRefs(m_targetStreamListener),
+                                     &abort);
+    
+  if (NS_FAILED(rv)) {
+    LOG_ERROR(("  DoContent failed"));
+    
+    // Unset the RETARGETED_DOCUMENT_URI flag if we set it...
+    aChannel->SetLoadFlags(loadFlags);
+    m_targetStreamListener = nsnull;
+    return PR_FALSE;
+  }
+
+  if (abort) {
+    // Nothing else to do here -- aListener is handling it all.  Make
+    // sure m_targetStreamListener is null so we don't do anything
+    // after this point.
+    LOG(("  Listener has aborted the load"));
+    m_targetStreamListener = nsnull;
+  }
+
+  NS_ASSERTION(abort || m_targetStreamListener, "DoContent returned no listener?");
+
+  // aListener is handling the load from this point on.  
+  return PR_TRUE;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////////////
 // Implementation of nsURILoader
 ///////////////////////////////////////////////////////////////////////////////////////////////
 
 nsURILoader::nsURILoader()
 {
+#ifdef PR_LOGGING
+  if (!mLog) {
+    mLog = PR_NewLogModule("URILoader");
+  }
+#endif
 }
 
 nsURILoader::~nsURILoader()
 {
 }
 
-NS_IMPL_ADDREF(nsURILoader);
-NS_IMPL_RELEASE(nsURILoader);
+NS_IMPL_ADDREF(nsURILoader)
+NS_IMPL_RELEASE(nsURILoader)
 
 NS_INTERFACE_MAP_BEGIN(nsURILoader)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIURILoader)
@@ -583,6 +801,16 @@ NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel *channel,
 {
   NS_ENSURE_ARG_POINTER(channel);
 
+#ifdef PR_LOGGING
+  if (LOG_ENABLED()) {
+    nsCOMPtr<nsIURI> uri;
+    channel->GetURI(getter_AddRefs(uri));
+    nsCAutoString spec;
+    uri->GetAsciiSpec(spec);
+    LOG(("nsURILoader::OpenURI for %s", spec.get()));
+  }
+#endif
+  
   // Let the window context's uriListener know that the open is starting.  This
   // gives that window a chance to abort the load process.
   nsCOMPtr<nsIURIContentListener> winContextListener(do_GetInterface(aWindowContext));
@@ -594,6 +822,7 @@ NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel *channel,
       winContextListener->OnStartURIOpen(uri, &doAbort);
          
       if (doAbort) {
+        LOG(("  OnStartURIOpen aborted load"));
         return NS_OK;
       }
     }   
@@ -602,7 +831,7 @@ NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel *channel,
   // we need to create a DocumentOpenInfo object which will go ahead and open
   // the url and discover the content type....
   nsCOMPtr<nsDocumentOpenInfo> loader =
-    new nsDocumentOpenInfo(aWindowContext, aIsContentPreferred);
+    new nsDocumentOpenInfo(aWindowContext, aIsContentPreferred, this);
 
   if (!loader) return NS_ERROR_OUT_OF_MEMORY;
 
@@ -736,159 +965,3 @@ nsresult nsURILoader::SetupLoadCookie(nsISupports * aWindowContext,
   return rv;
 }
 
-PRBool nsURILoader::ShouldHandleContent(nsIURIContentListener * aCntListener, 
-                                        const char * aContentType,
-                                        PRBool aIsContentPreferred,
-                                        char ** aContentTypeToUse)
-{
-  PRBool foundContentHandler = PR_FALSE;
-  if (aIsContentPreferred) {
-    aCntListener->IsPreferred(aContentType,
-                              aContentTypeToUse, 
-                              &foundContentHandler);
-  } else {
-    aCntListener->CanHandleContent(aContentType, PR_FALSE,
-                                   aContentTypeToUse, 
-                                   &foundContentHandler);
-  }
-  return foundContentHandler;
-} 
-
-NS_IMETHODIMP nsURILoader::DispatchContent(const char * aContentType,
-                                           PRBool aIsContentPreferred,
-                                           nsIRequest *request, 
-                                           nsISupports * aCtxt, 
-                                           nsIURIContentListener * aContentListener,
-                                           nsISupports * aSrcWindowContext,
-                                           char ** aContentTypeToUse,
-                                           nsIURIContentListener ** aContentListenerToUse,
-                                           PRBool * aAbortProcess)
-{
-  NS_ENSURE_ARG_POINTER(aContentType);
-  NS_ENSURE_ARG_POINTER(request);
-  NS_PRECONDITION(aAbortProcess, "Null out param!");
-  NS_PRECONDITION(aContentListenerToUse, "Null out param!");
-  NS_PRECONDITION(aContentTypeToUse, "Null out param!");
-
-  // okay, now we've discovered the content type. We need to do the
-  // following:
-  // (1) We always start with the original content listener (if any)
-  //     that originated the request and then ask if it can handle the
-  //     content.
-  // (2) if it can't, we'll move on to the registered content
-  //     listeners and give them a crack at handling the content.
-  // (3) if we cannot find a registered content lister to handle the
-  //     type, then we move on to phase II which is to try to find a
-  //     content handler in the registry for the content type.
-  //     hitting this phase usually means we'll be creating a new
-  //     window or handing off to an external application.
-
-  nsresult rv = NS_OK;
-
-  nsCOMPtr<nsIURIContentListener> listenerToUse = aContentListener;
-
-  PRBool foundContentHandler = PR_FALSE;
-  if (listenerToUse)
-    foundContentHandler = ShouldHandleContent(listenerToUse,
-                                              aContentType, 
-                                              aIsContentPreferred,
-                                              aContentTypeToUse);
-                                            
-
-  // if it can't handle the content, scan through the list of
-  // registered listeners
-  if (!foundContentHandler)
-  {
-    PRInt32 count = m_listeners.Count();
-    PRInt32 i;
-    
-    // keep looping until we get a content listener back
-    for(i = 0; i < count && !foundContentHandler; i++)
-    {
-      //nsIURIContentListener's aren't refcounted.
-      nsWeakPtr weakListener = m_listeners[i];
-      nsCOMPtr<nsIURIContentListener> listener(do_QueryReferent(weakListener));
-      
-      if (listener)
-      {
-        foundContentHandler = ShouldHandleContent(listener,
-                                                  aContentType, 
-                                                  aIsContentPreferred,
-                                                  aContentTypeToUse);
-        if (foundContentHandler) {
-          listenerToUse = listener;
-        }
-      } else {
-        // remove from the listener list, reset i and update count
-        m_listeners.RemoveObjectAt(i--);
-        --count;
-      }
-    } // for loop
-  } // if we can't handle the content
-
-
-  if (foundContentHandler && listenerToUse)
-  {
-    *aContentListenerToUse = listenerToUse;
-    NS_IF_ADDREF(*aContentListenerToUse);
-    return rv;
-  }
-
-  // Try to find a content listener, that had not yet the chance to register,
-  // as it is contained in a not-yet-loaded module, but which has registered a contract ID.
-  nsCOMPtr<nsICategoryManager> catman = do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-  if (catman)
-  {
-    // see if someone has registered a content listener for aContentType with the category manager...
-    nsXPIDLCString contractidString;
-    rv = catman->GetCategoryEntry(NS_CONTENT_LISTENER_CATEGORYMANAGER_ENTRY,
-                                  aContentType, getter_Copies(contractidString));
-    if (NS_SUCCEEDED(rv) && contractidString.get())
-    {
-      nsCOMPtr<nsIURIContentListener> listener;
-      listener = do_CreateInstance(contractidString.get(), &rv);
-      if (NS_SUCCEEDED(rv)) // we did indeed have a content listener for this type!! yippee...
-      {
-        foundContentHandler = ShouldHandleContent(listener, aContentType, aIsContentPreferred, aContentTypeToUse);
-
-        if (foundContentHandler && listener)
-        {
-          *aContentListenerToUse = listener;
-          NS_IF_ADDREF(*aContentListenerToUse);
-          return rv;
-        }
-      } // if we were able to create a uri content listener...
-    } // if we got a valid contract ID
-  } // if we can get the category manager....
- 
-  // no registered content listeners to handle this type!!! so go to the register 
-  // and get a registered nsIContentHandler for our content type. Hand it off 
-  // to them...
-  // eventually we want to hit up the category manager so we can allow people to
-  // over ride the default content type handlers....for now...i'm skipping that part.
-
-  nsCAutoString handlerContractID (NS_CONTENT_HANDLER_CONTRACTID_PREFIX);
-  handlerContractID += aContentType;
-  
-  nsCOMPtr<nsIContentHandler> aContentHandler;
-  aContentHandler = do_CreateInstance(handlerContractID.get(), &rv);
-  if (NS_SUCCEEDED(rv)) // we did indeed have a content handler for this type!! yippee...
-  {
-    rv = aContentHandler->HandleContent(aContentType, "view", aSrcWindowContext, request);
-    if (rv != NS_ERROR_WONT_HANDLE_CONTENT) {
-      *aAbortProcess = PR_TRUE;
-
-      // The content handler has unexpectedly failed.  Cancel the request
-      // just in case the handler didn't...
-      if (NS_FAILED(rv)) {
-        request->Cancel(rv);
-      }
-    }
-    // If WONT_HANDLE_CONTENT is returned -- do not abort.  Just return the
-    // failure and keep looking for another consumer (ie. the unknown
-    // content handler)...
-  }
-  
-  return rv;
-}
-
