@@ -38,10 +38,8 @@
 #include "nsIModule.h"
 #include "xcDll.h"
 #include "nsHashtable.h"
-#include "nsHashtableEnumerator.h"
 #include "nsXPIDLString.h"
 #include "nsCRT.h"
-
 #include "nsIObserverService.h"
 
 #if defined(XP_MAC)  // sdagley dougt fix
@@ -50,44 +48,28 @@
 #include "nsILocalFileMac.h"
 #endif
 
-#define PRINT_CRITICAL_ERROR_TO_SCREEN 1
-#define USE_REGISTRY 1
-#undef OBSOLETE_MODULE_LOADING
-
-#ifdef OBSOLETE_MODULE_LOADING
-#include "nsObsoleteModuleLoading.h"
-#endif
-
-// Logging of debug output
-#ifdef MOZ_LOGGING
-#define FORCE_PR_LOG /* Allow logging in the release build */
-#endif
 #include "prlog.h"
 extern PRLogModuleInfo *nsComponentManagerLog;
 
-nsNativeComponentLoader::nsNativeComponentLoader() :
-    mRegistry(nsnull), mCompMgr(nsnull), mDllStore(nsnull)
-{
-    NS_INIT_REFCNT();
-}
-
 static PRBool PR_CALLBACK
-nsDll_Destroy(nsHashKey *aKey, void *aData, void* closure)
+DLLStore_Destroy(nsHashKey *aKey, void *aData, void* closure)
 {
     nsDll* entry = NS_STATIC_CAST(nsDll*, aData);
     delete entry;
     return PR_TRUE;
 }
 
-nsNativeComponentLoader::~nsNativeComponentLoader()
+nsNativeComponentLoader::nsNativeComponentLoader() :
+    mCompMgr(nsnull),
+    mLoadedDependentLibs(16, PR_TRUE),
+    mDllStore(nsnull, nsnull, DLLStore_Destroy, 
+              nsnull, 256, PR_TRUE)
 {
-    mRegistry = nsnull;
-    mCompMgr = nsnull;
-
-    delete mDllStore;
 }
-    
-NS_IMPL_ISUPPORTS1(nsNativeComponentLoader, nsIComponentLoader);
+
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsNativeComponentLoader, 
+                              nsIComponentLoader,
+                              nsINativeComponentLoader)
 
 NS_IMETHODIMP
 nsNativeComponentLoader::GetFactory(const nsIID & aCID,
@@ -104,8 +86,7 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
 
     /* Should this all live in xcDll? */
     nsDll *dll;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(nsnull, aLocation, &mod, &size, &dll);
+    rv = CreateDll(nsnull, aLocation, &dll);
     if (NS_FAILED(rv))
         return rv;
 
@@ -113,13 +94,17 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
         return NS_ERROR_OUT_OF_MEMORY;
 
     if (!dll->IsLoaded()) {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
                ("nsNativeComponentLoader: loading \"%s\"",
-                dll->GetDisplayPath()));
-
+                displayPath.get()));
+#endif
         if (!dll->Load()) {
 
-            PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
+            PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
                    ("nsNativeComponentLoader: load FAILED"));
         
             char errorMsg[1024] = "<unknown; can't get error from NSPR>";
@@ -140,14 +125,8 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
         return rv;      // XXX translate error code?
     
     rv = GetFactoryFromModule(dll, aCID, _retval);
-#ifdef OBSOLETE_MODULE_LOADING
-    if (NS_FAILED(rv)) {
-        if (rv == NS_ERROR_FACTORY_NOT_LOADED) {
-            rv = GetFactoryFromNSGetFactory(dll, aCID, serviceMgr, _retval);
-        }
-    }
-#endif
-    PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
+
+    PR_LOG(nsComponentManagerLog, NS_SUCCEEDED(rv) ? PR_LOG_DEBUG : PR_LOG_ERROR,
            ("nsNativeComponentLoader: Factory creation %s for %s",
             (NS_SUCCEEDED(rv) ? "succeeded" : "FAILED"),
             aLocation));
@@ -168,79 +147,10 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
 NS_IMETHODIMP
 nsNativeComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
 {
-    nsresult rv;
-
     mCompMgr = aCompMgr;
-    mRegistry = do_QueryInterface(aReg);
-    if (!mCompMgr || !mRegistry)
+    if (!mCompMgr)
         return NS_ERROR_INVALID_ARG;
 
-    rv = mRegistry->GetSubtree(nsIRegistry::Common, xpcomComponentsKeyName,
-                               &mXPCOMKey);
-    if (NS_FAILED(rv))
-        return rv;
-
-    if (!mDllStore) {
-        mDllStore = new nsObjectHashtable(nsnull, nsnull, // never copy
-                                          nsDll_Destroy, nsnull,
-                                          256, /* Thead Safe */ PR_TRUE);
-        if (!mDllStore)
-            return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-
-    // Read in all dll entries and populate the mDllStore
-    nsCOMPtr<nsIEnumerator> dllEnum;
-    rv = mRegistry->EnumerateSubtrees( mXPCOMKey, getter_AddRefs(dllEnum));
-    if (NS_FAILED(rv)) return rv;
-
-    rv = dllEnum->First();
-    for (; NS_SUCCEEDED(rv) && (dllEnum->IsDone() != NS_OK); (rv = dllEnum->Next()))
-    {
-        nsCOMPtr<nsISupports> base;
-        rv = dllEnum->CurrentItem(getter_AddRefs(base));
-        if (NS_FAILED(rv))  continue;
-
-        // Get specific interface.
-        nsIID nodeIID = NS_IREGISTRYNODE_IID;
-        nsCOMPtr<nsIRegistryNode> node;
-        rv = base->QueryInterface( nodeIID, getter_AddRefs(node) );
-        if (NS_FAILED(rv)) continue;
-
-        // Get library name
-        nsXPIDLCString library;
-        rv = node->GetNameUTF8(getter_Copies(library));
-
-        if (NS_FAILED(rv)) continue;
-
-        char* uLibrary;
-        char* eLibrary = (char*)library.operator const char*();
-        PRUint32 length = strlen(eLibrary);
-        rv = mRegistry->UnescapeKey((PRUint8*)eLibrary, 1, &length, (PRUint8**)&uLibrary);
-        
-        if (NS_FAILED(rv)) continue;
-
-        if (uLibrary == nsnull)
-            uLibrary = eLibrary;
-
-        // Get key associated with library
-        nsRegistryKey libKey;
-        rv = node->GetKey(&libKey);
-        if (NS_SUCCEEDED(rv)) //  Cannot continue here, because we have to free unescape
-        {
-            // Create nsDll with this name
-            nsDll *dll = NULL;
-            PRInt64 lastModTime;
-            PRInt64 fileSize;
-            GetRegistryDllInfo(libKey, &lastModTime, &fileSize);
-            rv = CreateDll(NULL, uLibrary, &lastModTime, &fileSize, &dll);
-        }
-        if (uLibrary && (uLibrary != eLibrary))
-            nsMemory::Free(uLibrary);
-
-        if (NS_FAILED(rv)) continue;
-    }
-
     return NS_OK;
 }
 
@@ -250,13 +160,13 @@ nsNativeComponentLoader::AutoRegisterComponents(PRInt32 aWhen,
 {
 #ifdef DEBUG
     /* do we _really_ want to print this every time? */
-    printf("nsNativeComponentLoader: autoregistering begins.\n");
+    fprintf(stderr, "nsNativeComponentLoader: autoregistering begins.\n");
 #endif
 
     nsresult rv = RegisterComponentsInDir(aWhen, aDirectory);
 
 #ifdef DEBUG
-    printf("nsNativeComponentLoader: autoregistering %s\n",
+    fprintf(stderr, "nsNativeComponentLoader: autoregistering %s\n",
            NS_FAILED(rv) ? "FAILED" : "succeeded");
 #endif
 
@@ -352,25 +262,6 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     {
         rv = mobj->CanUnload(nsComponentManagerImpl::gComponentManager, &canUnload);
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        // Try the old method of module unloading
-        nsCanUnloadProc proc = (nsCanUnloadProc)dll->FindSymbol("NSCanUnload");
-        if (proc)
-        {
-            canUnload = proc(serviceMgr);
-            rv = NS_OK;    // No error status returned by call.
-        }
-        else
-        {
-            PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
-                   ("nsNativeComponentLoader: Unload cant get nsIModule or NSCanUnload for %s",
-                    dll->GetDisplayPath()));
-            return rv;
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
 
     mobj = nsnull; // Release our reference to the module object
     // When shutting down, whether we can unload the dll or not,
@@ -383,9 +274,14 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     // Check error status on CanUnload() call
     if (NS_FAILED(rv))
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
                ("nsNativeComponentLoader: nsIModule::CanUnload() returned error for %s.",
-                dll->GetDisplayPath()));
+                displayPath.get()));
+#endif
         return rv;
     }
 
@@ -393,9 +289,15 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     {
         if (dllMarkedForUnload)
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, 
-                   ("nsNativeComponentLoader: + Unloading \"%s\".", dll->GetDisplayPath()));
-#if 0
+                   ("nsNativeComponentLoader: + Unloading \"%s\".", displayPath.get()));
+#endif
+
+#ifdef DEBUG_dougt
             // XXX dlls aren't counting their outstanding instances correctly
             // XXX hence, dont unload until this gets enforced.
             rv = dll->Unload();
@@ -403,14 +305,24 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
         }
         else
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-                   ("nsNativeComponentLoader: Ready for unload \"%s\".", dll->GetDisplayPath()));
+                   ("nsNativeComponentLoader: Ready for unload \"%s\".", displayPath.get()));
+#endif
         }
     }
     else
     {
-        PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
-               ("nsNativeComponentLoader: NOT ready for unload %s", dll->GetDisplayPath()));
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
+        PR_LOG(nsComponentManagerLog, PR_LOG_WARNING, 
+               ("nsNativeComponentLoader: NOT ready for unload %s", displayPath.get()));
+#endif
         rv = NS_ERROR_FAILURE;
     }
     return rv;
@@ -465,15 +377,20 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
         return NS_ERROR_FAILURE;
     }
 
+#ifdef PR_LOGGING
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
+    
     PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-           ("nsNativeComponentLoader: Loaded \"%s\".", dll->GetDisplayPath()));
+           ("nsNativeComponentLoader: Loaded \"%s\".", displayPath.get()));
+#endif
 
     // Tell the module to self register
+    nsCOMPtr<nsIFile> fs;
     nsCOMPtr<nsIModule> mobj;
     res = dll->GetModule(mCompMgr, getter_AddRefs(mobj));
     if (NS_SUCCEEDED(res))
     {
-        nsCOMPtr<nsIFile> fs;
         /*************************************************************
          * WARNING: Why are use introducing 'res2' here and then     *
          * later assigning it to 'res' rather than just using 'res'? *
@@ -484,46 +401,47 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
          *************************************************************/
         nsresult res2 = dll->GetDllSpec(getter_AddRefs(fs));    // don't change 'res2' -- see warning, above
         if (NS_SUCCEEDED(res2)) {
+            // in the case of re-registering a component, we want to remove 
+            // any optional data that this file may have had.
+            AddDependentLibrary(fs, nsnull);
+
             res = mobj->RegisterSelf(mCompMgr, fs, registryLocation,
                                      nativeComponentType);
         }
         else
         {
             res = res2;         // don't take this out -- see warning, above
+
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
             PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
                    ("nsNativeComponentLoader: dll->GetDllSpec() on %s FAILED.",
-                    dll->GetDisplayPath()));
+                    displayPath.get()));
+#endif
         }
         mobj = NULL;    // Force a release of the Module object before unload()
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        res = NS_ERROR_NO_INTERFACE;
-        nsRegisterProc regproc = (nsRegisterProc)dll->FindSymbol("NSRegisterSelf");
-        if (regproc)
-        {
-            // Call the NSRegisterSelfProc to enable dll registration
-            res = regproc(serviceMgr, registryLocation);
-            PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
-                   ("nsNativeComponentLoader: %s using OBSOLETE NSRegisterSelf()",
-                    dll->GetDisplayPath()));
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
 
     // Update the timestamp and size of the dll in registry
     // Don't enter deferred modules in the registry, because it might only be
     // able to register on some later autoreg, after another component has been
     // installed.
     if (res != NS_ERROR_FACTORY_REGISTER_AGAIN) {
-        dll->Sync();
+        PRInt64 modTime;
+        if (!fs)
+            return res;
         
-        PRInt64 modTime, size;
-        dll->GetLastModifiedTime(&modTime);
-        dll->GetSize(&size);
-
-        SetRegistryDllInfo(registryLocation, &modTime, &size);
+        fs->GetLastModifiedTime(&modTime);
+        nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr);
+        if (!manager)
+            return NS_ERROR_FAILURE;
+        
+        nsCOMPtr<nsIFile> fs;
+        res = dll->GetDllSpec(getter_AddRefs(fs));
+        if (NS_FAILED(res)) return res;
+        
+        manager->SaveFileInfo(fs, registryLocation, modTime);
     }
 
     return res;
@@ -534,7 +452,7 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
 //
 
 #if defined(MOZ_DEMANGLE_SYMBOLS)
-#include "nsTraceRefcnt.h" // for nsTraceRefcnt::DemangleSymbol()
+#include "nsTraceRefcntImpl.h" // for nsTraceRefcntImpl::DemangleSymbol()
 #endif
 
 nsresult 
@@ -568,7 +486,7 @@ nsNativeComponentLoader::DumpLoadError(nsDll *dll,
         
         char demangled[4096] = "\0";
         
-        nsTraceRefcnt::DemangleSymbol(symbol.get(),demangled,sizeof(demangled));
+        nsTraceRefcntImpl::DemangleSymbol(symbol.get(),demangled,sizeof(demangled));
         
         if (demangled && *demangled != '\0')
             demangledSymbol = demangled;
@@ -589,25 +507,25 @@ nsNativeComponentLoader::DumpLoadError(nsDll *dll,
         }    
     }
 #endif // MOZ_DEMANGLE_SYMBOLS
-    
-    // Do NSPR log
-    PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
-           ("nsNativeComponentLoader: %s(%s) Load FAILED with error:%s", 
-            aCallerName,
-            dll->GetDisplayPath(), 
-            errorMsg.get()));
-    
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
 
-    // Dump to screen if needed
-#ifdef PRINT_CRITICAL_ERROR_TO_SCREEN
-    printf("**************************************************\n"
-           "nsNativeComponentLoader: %s(%s) Load FAILED with error: %s\n"
-           "**************************************************\n",
-           aCallerName,
-           dll->GetDisplayPath(), 
-           errorMsg.get());
+#ifdef DEBUG
+    fprintf(stderr, 
+            "nsNativeComponentLoader: %s(%s) Load FAILED with error: %s\n", 
+            aCallerName,
+            displayPath.get(), 
+            errorMsg.get());
 #endif
 
+    // Do NSPR log
+#ifdef PR_LOGGING
+    PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
+           ("nsNativeComponentLoader: %s(%s) Load FAILED with error: %s", 
+            aCallerName,
+            displayPath.get(), 
+            errorMsg.get()));
+#endif
     return NS_OK;
 }
 
@@ -630,8 +548,13 @@ nsNativeComponentLoader::SelfUnregisterDll(nsDll *dll)
     res = dll->GetModule(mCompMgr, getter_AddRefs(mobj));
     if (NS_SUCCEEDED(res))
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
-               ("nsNativeComponentLoader: %s using nsIModule to unregister self.", dll->GetDisplayPath()));
+               ("nsNativeComponentLoader: %s using nsIModule to unregister self.", displayPath.get()));
+#endif
         nsCOMPtr<nsIFile> fs;
         res = dll->GetDllSpec(getter_AddRefs(fs));
         if (NS_FAILED(res)) return res;
@@ -647,19 +570,6 @@ nsNativeComponentLoader::SelfUnregisterDll(nsDll *dll)
         if (NS_FAILED(res)) return res;
         mobj->UnregisterSelf(mCompMgr, fs, registryName);
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        res = NS_ERROR_NO_INTERFACE;
-        nsUnregisterProc unregproc =
-            (nsUnregisterProc) dll->FindSymbol("NSUnregisterSelf");
-        if (unregproc)
-        {
-            // Call the NSUnregisterSelfProc to enable dll de-registration
-            res = unregproc(serviceMgr, dll->GetPersistentDescriptorString());
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
     return res;
 }
 
@@ -668,8 +578,11 @@ nsNativeComponentLoader::AutoUnregisterComponent(PRInt32 when,
                                                  nsIFile *component,
                                                  PRBool *unregistered)
 {
+
     nsresult rv = NS_ERROR_FAILURE;
 
+    *unregistered = PR_FALSE;
+
     nsXPIDLCString persistentDescriptor;
     // what I want to do here is QI for a Component Registration Manager.  Since this 
     // has not been invented yet, QI to the obsolete manager.  Kids, don't do this at home.
@@ -695,19 +608,33 @@ nsNativeComponentLoader::AutoUnregisterComponent(PRInt32 when,
     }
 
     nsDll *dll = NULL;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(component, persistentDescriptor, &mod, &size, &dll);
+    rv = CreateDll(component, persistentDescriptor, &dll);
     if (NS_FAILED(rv) || dll == NULL) return rv;
 
     rv = SelfUnregisterDll(dll);
 
-    // Remove any autoreg info about this dll
-    if (NS_SUCCEEDED(rv))
-        RemoveRegistryDllInfo(persistentDescriptor);
+#ifdef PR_LOGGING
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
 
-    PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
+    PR_LOG(nsComponentManagerLog, NS_SUCCEEDED(rv) ? PR_LOG_DEBUG : PR_LOG_ERROR,
            ("nsNativeComponentLoader: AutoUnregistration for %s %s.",
-            (NS_FAILED(rv) ? "FAILED" : "succeeded"), dll->GetDisplayPath()));
+            (NS_FAILED(rv) ? "FAILED" : "succeeded"), displayPath.get()));
+#endif
+
+    if (NS_FAILED(rv))
+        return rv;
+
+    // Remove any autoreg info about this dll
+    nsCStringKey key(persistentDescriptor);
+    mDllStore.RemoveAndDelete(&key);
+    
+    nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr);
+    NS_ASSERTION(manager, "Something is terribly wrong");
+
+    manager->RemoveFileInfo(component, nsnull);
+
+    *unregistered = PR_TRUE;
     return rv;
 }
 
@@ -720,6 +647,8 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
     if (!registered)
         return NS_ERROR_NULL_POINTER;
 
+    *registered = PR_FALSE;
+
     /* this should be a pref or registry entry, or something */
     static const char *ValidDllExtensions[] = {
         ".dll",     /* Windows */
@@ -772,24 +701,23 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
     }
 
 #else
-    char *leafName = NULL;
-    rv = component->GetLeafName(&leafName);
+    nsCAutoString leafName;
+    rv = component->GetNativeLeafName(leafName);
     if (NS_FAILED(rv)) return rv;
-    int flen = PL_strlen(leafName);
+    int flen = leafName.Length();
     for (int i=0; ValidDllExtensions[i] != NULL; i++)
     {
         int extlen = PL_strlen(ValidDllExtensions[i]);
             
         // Does fullname end with this extension
         if (flen >= extlen &&
-            !PL_strcasecmp(&(leafName[flen - extlen]), ValidDllExtensions[i])
+            !PL_strcasecmp(leafName.get() + (flen - extlen), ValidDllExtensions[i])
             )
         {
             validExtension = PR_TRUE;
             break;
         }
     }
-    if (leafName) nsMemory::Free(leafName);
 #endif
         
     if (validExtension == PR_FALSE)
@@ -810,29 +738,25 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
 
     // Get the registry representation of the dll, if any
     nsDll *dll;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(component, persistentDescriptor, &mod, &size, &dll);
+    rv = CreateDll(component, persistentDescriptor, &dll);
     if (NS_FAILED(rv))
         return rv;
 
     if (dll != NULL)
     {
-        // Make sure the dll is OK
-        if (dll->GetStatus() != NS_OK)
-        {
-            PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
-                   ("nsNativeComponentLoader: + nsDll not NS_OK \"%s\". Skipping...",
-                    dll->GetDisplayPath()));
-            return NS_ERROR_FAILURE;
-        }
-            
         // We already have seen this dll. Check if this dll changed
         if (!dll->HasChanged())
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+            
             // Dll hasn't changed. Skip.
-            PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
+            PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, 
                    ("nsNativeComponentLoader: + nsDll not changed \"%s\". Skipping...",
-                    dll->GetDisplayPath()));
+                    displayPath.get()));
+#endif
+            *registered = PR_TRUE;
             return NS_OK;
         }
 
@@ -857,9 +781,7 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
             nsCOMPtr<nsIFile> dllSpec;
             if (NS_SUCCEEDED(dll->GetDllSpec(getter_AddRefs(dllSpec))) && dllSpec)
             {
-              nsXPIDLCString dllLeafName;
-              dllSpec->GetLeafName(getter_Copies(dllLeafName));
-              fileName.AssignWithConversion(dllLeafName);
+              dllSpec->GetLeafName(fileName);
             }
             
             // this string can't come from a string bundle, because we
@@ -885,19 +807,28 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
                 // THIS IS THE WORST SITUATION TO BE IN.
                 // Dll doesn't want to be unloaded. Cannot re-register
                 // this dll.
-                PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
+#ifdef PR_LOGGING
+                nsXPIDLCString displayPath;
+                dll->GetDisplayPath(displayPath);
+                
+                PR_LOG(nsComponentManagerLog, PR_LOG_WARNING,
                        ("nsNativeComponentLoader: *** Dll already loaded. "
                         "Cannot unload either. Hence cannot re-register "
-                        "\"%s\". Skipping...", dll->GetDisplayPath()));
+                        "\"%s\". Skipping...", displayPath.get()));
+#endif
                 return rv;
             }
             else {
                                 // dll doesn't have a CanUnload proc. Guess it is
                                 // ok to unload it.
                 dll->Unload();
-                PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
+#ifdef PR_LOGGING
+                nsXPIDLCString displayPath;
+                dll->GetDisplayPath(displayPath);
+                PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, 
                        ("nsNativeComponentLoader: + Unloading \"%s\". (no CanUnloadProc).",
-                        dll->GetDisplayPath()));
+                        displayPath.get()));
+#endif
             }
                 
         } // dll isloaded
@@ -908,9 +839,13 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
             // We went through all the above to make sure the dll
             // is unloaded. And here we are with the dll still
             // loaded. Whoever taught dp programming...
-            PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+            PR_LOG(nsComponentManagerLog, PR_LOG_WARNING,
                    ("nsNativeComponentLoader: Dll still loaded. Cannot re-register "
-                    "\"%s\". Skipping...", dll->GetDisplayPath()));
+                    "\"%s\". Skipping...", displayPath.get()));
+#endif
             return NS_ERROR_FAILURE;
         }
     } // dll != NULL
@@ -920,10 +855,10 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
         // It is ok to do this even if the creation of nsDll
         // didnt succeed. That way we wont do this again
         // when we encounter the same dll.
-        dll = new nsDll(persistentDescriptor);
+        dll = new nsDll(component, this);
         if (dll == NULL)
             return NS_ERROR_OUT_OF_MEMORY;
-        mDllStore->Put(&key, (void *) dll);
+        mDllStore.Put(&key, (void *) dll);
     } // dll == NULL
         
     // Either we are seeing the dll for the first time or the dll has
@@ -936,22 +871,34 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
         if (res == NS_ERROR_FACTORY_REGISTER_AGAIN) {
             /* defer for later loading */
             mDeferredComponents.AppendElement(dll);
+            *registered = PR_TRUE;
             return NS_OK;
         } else {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
                    ("nsNativeComponentLoader: Autoregistration FAILED for "
-                    "\"%s\". Skipping...", dll->GetDisplayPath()));
+                    "\"%s\". Skipping...", displayPath.get()));
+#endif
             return NS_ERROR_FACTORY_NOT_REGISTERED;
         }
     }
     else
     {
-        PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
+        PR_LOG(nsComponentManagerLog, PR_LOG_WARNING,
                ("nsNativeComponentLoader: Autoregistration Passed for "
-                "\"%s\".", dll->GetDisplayPath()));
+                "\"%s\".", displayPath.get()));
+#endif
         // Marking dll along with modified time and size in the
         // registry happens at PlatformRegister(). No need to do it
         // here again.
+        *registered = PR_TRUE;
     }
     return NS_OK;
 }
@@ -971,7 +918,7 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
     for (int i = mDeferredComponents.Count() - 1; i >= 0; i--) {
         nsDll *dll = NS_STATIC_CAST(nsDll *, mDeferredComponents[i]);
         nsresult rv = SelfRegisterDll(dll,
-                                      dll->GetRegistryLocation(),
+                                      nsnull,
                                       PR_TRUE);
         if (rv != NS_ERROR_FACTORY_REGISTER_AGAIN) {
             if (NS_SUCCEEDED(rv))
@@ -979,13 +926,14 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
             mDeferredComponents.RemoveElementAt(i);
         }
     }
-
+#ifdef DEBUG
     if (*aRegistered)
         fprintf(stderr, "nNCL: registered deferred, %d left\n",
                 mDeferredComponents.Count());
     else
         fprintf(stderr, "nNCL: didn't register any components, %d left\n",
                 mDeferredComponents.Count());
+#endif
     /* are there any fatal errors? */
     return NS_OK;
 }
@@ -993,8 +941,10 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
 nsresult
 nsNativeComponentLoader::OnRegister(const nsIID &aCID, const char *aType,
                                     const char *aClassName,
-                                    const char *aContractID, const char *aLocation,
-                                    PRBool aReplace, PRBool aPersist)
+                                    const char *aContractID, 
+                                    const char *aLocation,
+                                    PRBool aReplace, 
+                                    PRBool aPersist)
 {
     return NS_OK;
 }
@@ -1009,104 +959,10 @@ nsNativeComponentLoader::UnloadAll(PRInt32 aWhen)
     callData.when = aWhen;
 
     // Cycle through the dlls checking to see if they want to be unloaded
-    if (mDllStore) {
-        mDllStore->Enumerate(nsFreeLibraryEnum, &callData);
-    }
+    mDllStore.Enumerate(nsFreeLibraryEnum, &callData);
     return NS_OK;
 }
 
-nsresult
-nsNativeComponentLoader::GetRegistryDllInfo(const char *aLocation,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-    return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-
-    nsRegistryKey key;
-    rv = mRegistry->GetSubtreeRaw(mXPCOMKey, eLocation, &key);
-    if (NS_FAILED(rv)) return rv;
-
-    rv = GetRegistryDllInfo(key, lastModifiedTime, fileSize);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
-nsresult
-nsNativeComponentLoader::GetRegistryDllInfo(nsRegistryKey key,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    PRInt64 lastMod;
-    nsresult rv = mRegistry->GetLongLong(key, lastModValueName, &lastMod);
-    if (NS_FAILED(rv)) return rv;
-    *lastModifiedTime = lastMod;
-        
-    PRInt64 fsize;
-    rv = mRegistry->GetLongLong(key, fileSizeValueName, &fsize);
-    if (NS_FAILED(rv)) return rv;
-        *fileSize = fsize;
-    return NS_OK;
-}
-
-nsresult
-nsNativeComponentLoader::SetRegistryDllInfo(const char *aLocation,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-    nsRegistryKey key;
-    rv = mRegistry->GetSubtreeRaw(mXPCOMKey, eLocation, &key);
-    if (NS_FAILED(rv)) return rv;
-
-    rv = mRegistry->SetLongLong(key, lastModValueName, lastModifiedTime);
-    if (NS_FAILED(rv)) return rv;
-    rv = mRegistry->SetLongLong(key, fileSizeValueName, fileSize);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
-nsresult
-nsNativeComponentLoader::RemoveRegistryDllInfo(const char *aLocation)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-    rv = mRegistry->RemoveSubtree(mXPCOMKey, eLocation);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
 //
 // CreateDll
 // The only way to create a dll or get it from the dll cache. This will
@@ -1126,8 +982,8 @@ nsNativeComponentLoader::RemoveRegistryDllInfo(const char *aLocation)
 //    registry along with its    lastModTime and fileSize.
 //    {NULL, rel:libpref.so, 8985659, 20987}
 nsresult
-nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
-                                   PRInt64 *modificationTime, PRInt64 *fileSize,
+nsNativeComponentLoader::CreateDll(nsIFile *aSpec, 
+                                   const char *aLocation,
                                    nsDll **aDll)
 {
     nsDll *dll;
@@ -1136,7 +992,7 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
     nsresult rv;
 
     nsCStringKey key(aLocation);
-    dll = (nsDll *)mDllStore->Get(&key);
+    dll = (nsDll *)mDllStore.Get(&key);
     if (dll)
     {
         *aDll = dll;
@@ -1145,19 +1001,14 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
 
     if (!aSpec)
     {
-        if (!nsCRT::strncmp(aLocation, XPCOM_LIB_PREFIX, 4)) {
-            dll = new nsDll(aLocation+4, 1 /* dumb magic flag */);
-            if (!dll) return NS_ERROR_OUT_OF_MEMORY;
-        } else {
-            // what I want to do here is QI for a Component Registration Manager.  Since this 
-            // has not been invented yet, QI to the obsolete manager.  Kids, don't do this at home.
-            nsCOMPtr<nsIComponentManagerObsolete> obsoleteManager = do_QueryInterface(mCompMgr, &rv);
-            if (obsoleteManager)
-                rv = obsoleteManager->SpecForRegistryLocation(aLocation,
-                                                              getter_AddRefs(spec));
-            if (NS_FAILED(rv))
-                return rv;
-        }
+        // what I want to do here is QI for a Component Registration Manager.  Since this 
+        // has not been invented yet, QI to the obsolete manager.  Kids, don't do this at home.
+        nsCOMPtr<nsIComponentManagerObsolete> obsoleteManager = do_QueryInterface(mCompMgr, &rv);
+        if (obsoleteManager)
+            rv = obsoleteManager->SpecForRegistryLocation(aLocation,
+                                                          getter_AddRefs(spec));
+        if (NS_FAILED(rv))
+            return rv;
     }
     else
     {
@@ -1166,24 +1017,13 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
 
     if (!dll)
     {
-        PR_ASSERT(modificationTime != NULL);
-        PR_ASSERT(fileSize != NULL);
-
-        PRInt64 zit = LL_Zero();
-        
-        if (LL_EQ(*modificationTime,zit) && LL_EQ(*fileSize,zit))
-        {
-            // Get the modtime and filesize from the registry
-            rv = GetRegistryDllInfo(aLocation, modificationTime, fileSize);
-        }
-        dll = new nsDll(spec, aLocation, modificationTime, fileSize);
+        dll = new nsDll(spec, this);
+        if (!dll)
+            return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    if (!dll)
-        return NS_ERROR_OUT_OF_MEMORY;
-
     *aDll = dll;
-    mDllStore->Put(&key, dll);
+    mDllStore.Put(&key, dll);
     return NS_OK;
 }
 
@@ -1203,33 +1043,35 @@ nsNativeComponentLoader::GetFactoryFromModule(nsDll *aDll, const nsCID &aCID,
                                   (void **)aFactory);
 }
 
-nsresult
-nsNativeComponentLoader::GetFactoryFromNSGetFactory(nsDll *aDll,
-                                                    const nsCID &aCID,
-                                                    nsIServiceManager *aServMgr,
-                                                    nsIFactory **aFactory)
+
+NS_IMETHODIMP
+nsNativeComponentLoader::AddDependentLibrary(nsIFile* aFile, const char* libName)
 {
-#ifdef OBSOLETE_MODULE_LOADING
-    nsFactoryProc getFactory =
-        (nsFactoryProc) aDll->FindSymbol("NSGetFactory");
+    nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr);
+    if (!manager) 
+    {
+        NS_WARNING("Something is terribly wrong");
+        return NS_ERROR_FAILURE;
+    }
 
-    if (!getFactory)
-        return NS_ERROR_FACTORY_NOT_LOADED;
+    // the native component loader uses the optional data
+    // to store a space delimited list of dependent library 
+    // names
+
+    if (!libName) 
+    {
+        manager->SetOptionalData(aFile, nsnull, nsnull);
+        return NS_OK;
+    }
+
+    nsXPIDLCString data;
+    manager->GetOptionalData(aFile, nsnull, getter_Copies(data));
+
+    if (!data.IsEmpty())
+        data.Append(NS_LITERAL_CSTRING(" "));
+
+    data.Append(nsDependentCString(libName));
     
-    PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-           ("nsNativeComponentLoader: %s using OBSOLETE NSGetFactory()\n",
-            aDll->GetDisplayPath()));
-
-    /*
-     * There was a time when CLSIDToContractID was used to get className
-     * and contractID, but that day is long past.  This code is not long
-     * for this earth, so we just pass nsnull.
-     */
-
-    return getFactory(aServMgr, aCID, nsnull /*className */,
-                      nsnull /* contractID */, aFactory);
-
-#else /* !OBSOLETE_MODULE_LOADING */
-    return NS_ERROR_FACTORY_NOT_LOADED;
-#endif /* OBSOLETE_MODULE_LOADING */
+    manager->SetOptionalData(aFile, nsnull, data);
+    return NS_OK;
 }
